from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.config import config
from mipac.errors.base import NotSupportVersion
from mipac.http import Route
from mipac.models.emoji import CustomEmoji
from mipac.models.note import NoteReaction
from mipac.types.meta import ILiteMeta
from mipac.types.note import INoteReaction
from mipac.utils.format import remove_dict_empty

if TYPE_CHECKING:
    from mipac.http import HTTPClient
    from mipac.manager.client import ClientManager


class ReactionActions(AbstractAction):
    def __init__(self, note_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__note_id: str | None = note_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def add(self, reaction: str, note_id: str | None = None) -> bool:
        """
        指定したnoteに指定したリアクションを付与します（内部用

        Parameters
        ----------
        reaction : str | None
            付与するリアクション名
        note_id : str | None
            付与対象のノートID

        Returns
        -------
        bool
            成功したならTrue,失敗ならFalse
        """
        note_id = note_id or self.__note_id

        data = remove_dict_empty({"noteId": note_id, "reaction": reaction})
        route = Route("POST", "/api/notes/reactions/create")
        res: bool = await self.__session.request(route, json=data, auth=True, lower=True)
        return bool(res)

    async def remove(self, note_id: str | None = None) -> bool:
        note_id = note_id or self.__note_id

        data = remove_dict_empty({"noteId": note_id})
        route = Route("POST", "/api/notes/reactions/delete")
        res: bool = await self.__session.request(route, json=data, auth=True, lower=True)
        return bool(res)

    async def get_reaction(
        self, reaction: str, note_id: str | None = None, *, limit: int = 10
    ) -> list[NoteReaction]:
        note_id = note_id or self.__note_id
        data = remove_dict_empty({"noteId": note_id, "limit": limit, "type": reaction})
        res: list[INoteReaction] = await self.__session.request(
            Route("POST", "/api/notes/reactions"),
            json=data,
            auth=True,
            lower=True,
        )
        return [NoteReaction(i, client=self.__client) for i in res]

    async def get_emoji_list(self) -> list[CustomEmoji]:
        if config.use_version >= 13:
            raise NotSupportVersion("Misskey v13以降では使用できません")

        data: ILiteMeta = await self.__session.request(
            Route("GET", "/api/meta"),
            json={"detail": False},
            auth=True,
            replace_list={"ToSUrl": "tos_url", "ToSTextUrl": "tos_text_url"},
        )
        return [CustomEmoji(i, client=self.__client) for i in data.get("emojis", [])]



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.models.lite.emoji import PartialCustomEmoji
from mipac.types.emoji import ICustomEmoji

if TYPE_CHECKING:
    from mipac.client import ClientManager

__all__ = ("CustomEmoji",)


class CustomEmoji(PartialCustomEmoji):
    def __init__(self, emoji: ICustomEmoji, *, client: ClientManager):
        super().__init__(emoji, client=client)
        self.__emoji: ICustomEmoji = emoji

    @property
    def id(self) -> str:
        return self.__emoji["id"]

    @property
    def aliases(self) -> list[str]:
        return self.__emoji["aliases"]

    @property
    def category(self) -> str:
        return self.__emoji["category"]

    @property
    def license(self) -> str | None:
        return self.__emoji["license"]

    @property
    def host(self) -> str | None:
        return self.__emoji["host"]

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, CustomEmoji) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from typing import TypedDict

from mipac.types.user import ILiteUser


class IPartialInviteCode(TypedDict):
    code: str


class IInviteCode(IPartialInviteCode):
    id: str
    expires_at: str | None
    created_at: str
    created_by: str | None
    used_by: ILiteUser | None
    used_at: str | None
    used: bool



from __future__ import annotations

from typing import TYPE_CHECKING, Generic, Literal, TypeVar

from mipac.abstract.model import AbstractModel
from mipac.models.lite.instance import LiteInstance
from mipac.types.emoji import ICustomEmojiLite
from mipac.types.user import IBadgeRole, ILiteUser

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.manager.user import UserManager

T = TypeVar("T", bound=IBadgeRole)
LUT = TypeVar("LUT", bound=ILiteUser)


class BadgeRole(AbstractModel, Generic[T]):
    def __init__(self, data: T, *, client: ClientManager) -> None:
        self._data: T = data
        self._client = client

    @property
    def name(self) -> str:
        return self._data["name"]

    @property
    def icon_url(self) -> str | None:
        return self._data["icon_url"]

    @property
    def display_order(self) -> int:
        return self._data["display_order"]


class LiteUser(AbstractModel, Generic[LUT]):
    __slots__ = ("_user", "__client")

    def __init__(self, user: LUT, *, client: ClientManager) -> None:
        self._user: LUT = user
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self._user["id"]

    @property
    def username(self) -> str:
        return self._user["username"]

    @property
    def host(self) -> str | None:
        host = self._user.get("host")
        return host if host else None

    @property
    def nickname(self) -> str:
        return self._user["name"]

    @property
    def online_status(
        self,
    ) -> Literal["online", "active", "offline", "unknown"]:
        return self._user["online_status"]

    @property
    def badge_roles(self) -> list[BadgeRole]:
        return [
            BadgeRole(data, client=self.__client) for data in self._user.get("badge_roles", [])
        ]

    @property
    def avatar_url(self) -> str:
        return self._user["avatar_url"]

    @property
    def avatar_blurhash(self) -> str:
        return self._user["avatar_blurhash"]

    @property
    def avatar_color(self) -> str | None:
        """
        Returns the average color of the avatar.
        Note: Since avatar_color is deprecated in v13,
        only None is returned for v13 instances.

        Returns
        -------
        str | None
            average color of the avatar
        """

        return self._user.get("avatar_color")

    @property
    def emojis(self) -> list[ICustomEmojiLite]:  # TODO: ちゃんとモデルにする
        """
        List of emoji included in nicknames, etc
        Note: emojis have been abolished since misskey v13

        Returns
        -------
        list[ICustomEmojiLite]
            List of emoji included in nicknames, etc
        """

        return self._user.get("emojis", [])

    @property
    def instance(self) -> LiteInstance | None:
        instance = self._user.get("instance")
        return LiteInstance(instance) if instance else None

    @property
    def api(self) -> UserManager:
        return self.__client._create_user_instance(self)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, LiteUser) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



import asyncio
import uuid
from urllib.parse import urlencode

import aiohttp

from mipac.utils.format import remove_dict_empty


class AuthClient:
    """
    Tokenの取得を手助けするクラス
    """

    def __init__(
        self,
        instance_uri: str,
        name: str,
        description: str,
        permissions: list[str] | None = None,
        *,
        icon: str | None = None,
        use_miauth: bool = False,
    ):
        """
        Parameters
        ----------
        instance_uri : str
            アプリケーションを作成したいインスタンスのURL
        name : str
            アプリケーションの名前
        description : str
            アプリケーションの説明
        permissions : Optional[list[str]], default=None
            アプリケーションが要求する権限
        icon: str | None, default=None
            アプリケーションのアイコン画像URL
        use_miauth: bool, default=False
            MiAuthを使用するか
        """
        if permissions is None:
            permissions = ["read:account"]
        self.__client_session = aiohttp.ClientSession()
        self.__instance_uri: str = instance_uri
        self.__name: str = name
        self.__description: str = description
        self.__permissions: list[str] = permissions
        self.__icon: str | None = icon
        self.__use_miauth: bool = use_miauth
        self.__session_token: uuid.UUID
        self.__secret: str

    async def get_auth_url(self) -> str:
        """
        認証に使用するURLを取得します

        Returns
        -------
        str
            認証に使用するURL
        """
        field = remove_dict_empty(
            {"name": self.__name, "description": self.__description, "icon": self.__icon}
        )
        if self.__use_miauth:
            field["permissions"] = self.__permissions
            query = urlencode(field)
            self.__session_token = uuid.uuid4()
            return f"{self.__instance_uri}/miauth/{self.__session_token}?{query}"
        else:
            field["permission"] = self.__permissions
            async with self.__client_session.post(
                f"{self.__instance_uri}/api/app/create", json=field
            ) as res:
                data = await res.json()
                self.__secret = data["secret"]
            async with self.__client_session.post(
                f"{self.__instance_uri}/api/auth/session/generate",
                json={"appSecret": self.__secret},
            ) as res:
                data = await res.json()
                self.__session_token = data["token"]
                return data["url"]

    async def wait_miauth(self) -> str:
        url = f"{self.__instance_uri}/api/miauth/{self.__session_token}/check"
        while True:
            async with self.__client_session.post(url) as res:
                data = await res.json()
                if data.get("ok") is True:
                    return data
            await asyncio.sleep(1)

    async def wait_oldauth(self) -> None:
        while True:
            async with self.__client_session.post(
                f"{self.__instance_uri}/api/auth/session/userkey",
                json={"appSecret": self.__secret, "token": self.__session_token},
            ) as res:
                data = await res.json()
                if data.get("error", {}).get("code") != "PENDING_SESSION":
                    break
            await asyncio.sleep(1)

    async def check_auth(self) -> str:
        """
        認証が完了するまで待機し、完了した場合はTokenを返します

        Returns
        -------
        str
            Token
        """
        if self.__use_miauth:
            data = await self.wait_miauth()
        else:
            data = await self.wait_oldauth()
        await self.__client_session.close()
        return data["token"] if self.__use_miauth else data["accessToken"]



from importlib import import_module
from typing import Union


class APIError(Exception):
    """APIのエラー"""

    def __init__(self, data: Union[dict, str], status: int):
        self.raw: Union[dict, str] = data
        self.status: int = status
        self.code: str | None = None
        self.id: str | None = None
        self.message: str | None = None
        if isinstance(data, dict):
            error = data.get("error", {})
            if isinstance(error, dict):
                self.code: str | None = error.get("code", "")
                self.id: str | None = error.get("id")
                self.message: str | None = error.get("message", "")
        super().__init__(f"{self.message}\nRaw error: {self.raw} " if self.message else self.raw)

    def raise_error(self):
        if not self.code:
            raise self
        if value := getattr(
            import_module("mipac.errors.errors"),
            "".join([i.capitalize() for i in self.code.split("_")]) + "Error",
            None,
        ):
            raise value(self.raw, self.status)
        else:
            raise self


class NotExistRequiredData(Exception):
    """クラスの中に必要なデータが不足している"""


class ParameterError(Exception):
    """引数に関するエラー"""


class NotSupportVersion(Exception):
    """サポートされていないバージョンのインスタンス"""


NotSupportVersionText = "ご利用のインスタンスのバージョンではサポートされていない機能です"



from __future__ import annotations

from typing import TYPE_CHECKING, Literal, TypedDict

if TYPE_CHECKING:
    from mipac.types.user import ILiteUser

__all__ = ("NoteReactionPayload",)

IReactionAcceptance = Literal["likeOnly", "likeOnlyForRemote"]


class NoteReactionPayload(TypedDict):
    id: str
    created_at: str
    user: ILiteUser
    type: str



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.mute import MuteActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class MuteManager(AbstractManager):
    def __init__(
        self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager
    ) -> None:
        self._user_id: str | None = user_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    @property
    def action(self) -> MuteActions:
        return MuteActions(user_id=self._user_id, session=self._session, client=self._client)



from typing import TypedDict

from mipac.types.user import ILiteUser


class IClip(TypedDict):
    id: str
    created_at: str
    last_clipped_at: str
    user_id: str
    user: ILiteUser
    name: str
    description: str | None
    is_public: bool
    is_favorited: bool
    favorited_count: int



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.lite.user import LiteUser
from mipac.types.follow import IFollowRequest

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.manager.follow import FollowRequestManager


class FollowRequest(AbstractModel):
    def __init__(self, follow_request: IFollowRequest, *, client: ClientManager) -> None:
        self.__follow_request = follow_request
        self.__client = client

    @property
    def id(self) -> str:
        return self.__follow_request["id"]

    @property
    def follower(self) -> LiteUser:
        return LiteUser(self.__follow_request["follower"], client=self.__client)

    @property
    def followee(self) -> LiteUser:
        return LiteUser(self.__follow_request["followee"], client=self.__client)

    @property
    def api(self) -> FollowRequestManager:
        return self.__client._create_user_instance(user=self.follower).follow.request

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, FollowRequest) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from __future__ import annotations
from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.http import HTTPClient
from mipac.actions.role import RoleActions

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class RoleManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> RoleActions:
        return RoleActions(session=self.__session, client=self.__client)



from .base import APIError


class AccessDeniedError(APIError):
    """アクセス権限がありません。"""


class AlreadyAddedError(APIError):
    """ """


class AlreadyBlockingError(APIError):
    """すでにブロックしています。"""


class AlreadyClippedError(APIError):
    """ """


class AlreadyExpiredError(APIError):
    """ """


class AlreadyFavoritedError(APIError):
    """既にお気に入り登録されています。"""


class AlreadyFollowingError(APIError):
    """すでにフォローしています。"""


class AlreadyInvitedError(APIError):
    """ """


class AlreadyLikedError(APIError):
    """すでにいいねをつけています。"""


class AlreadyMutingError(APIError):
    """すでにそのユーザーをミュートしています。"""


class AlreadyPinnedError(APIError):
    """指定されたノートは既にピン留めされています。"""


class AlreadyPromotedError(APIError):
    """ """


class AlreadyReactedError(APIError):
    """既にリアクションしています。"""


class AlreadyVotedError(APIError):
    """ """


class AvatarNotAnImageError(APIError):
    """アバター画像に、画像ではないファイルが指定されました。"""


class BannerNotAnImageError(APIError):
    """バナー画像に、画像ではないファイルが指定されました。"""


class BlockedError(APIError):
    """ユーザーにブロックされています。"""


class BlockeeIsYourselfError(APIError):
    """自分のブロックを解除しようとしました。"""


class BlockingError(APIError):
    """ユーザーをブロックしています。"""


class CannotCreateAlreadyExpiredPollError(APIError):
    """アンケートの期限の指定が誤っています。"""


class CannotRenoteToAPureRenoteError(APIError):
    """単純なRenoteを再度Renoteすることはできません。"""


class CannotReplyToAPureRenoteError(APIError):
    """単純なRenoteに返信することはできません。"""


class CannotReportTheAdminError(APIError):
    """管理者を通報しようとしました。"""


class CannotReportYourselfError(APIError):
    """自身を通報しようとしました。"""


class ContentRequiredError(APIError):
    """ """


class CredentialRequiredError(APIError):
    """クレデンシャル必須のエンドポイントにクレデンシャル無しでリクエストされました。"""


class FailedToResolveRemoteUserError(APIError):
    """リモートユーザーの検索に失敗しました。"""


class FolloweeIsYourselfError(APIError):
    """自分のフォローを解除しようとしました。"""


class FollowerIsYourselfError(APIError):
    """自分をフォロワー解除しようとしました。"""


class FollowRequestNotFoundError(APIError):
    """フォローリクエストがありません。"""


class ForbiddenError(APIError):
    """ """


class GroupAccessDeniedError(APIError):
    """ """


class GtlDisabledError(APIError):
    """グローバルタイムラインが無効になっています。"""


class HasChildFilesOrFoldersError(APIError):
    """フォルダーが空ではありません。"""


class InappropriateError(APIError):
    """不適切なコンテンツを含んでいる可能性があると判定されました。"""


class InternalErrorError(APIError):
    """サーバー内部で問題が発生しました。引き続き問題が発生する場合は管理者にお問い合わせください。"""


class InvalidChoiceError(APIError):
    """ """


class InvalidFileNameError(APIError):
    """ファイル名が不正です。"""


class InvalidParamError(APIError):
    """リクエストパラメータに誤りがあります。"""


class InvalidRegexpError(APIError):
    """正規表現が不正です。"""


class InvalidUrlError(APIError):
    """ """


class IsOwnerError(APIError):
    """ """


class LtlDisabledError(APIError):
    """ローカルタイムラインが無効になっています。"""


class MoSuchFileError(APIError):
    """ """


class MuteeIsYourselfError(APIError):
    """自分に対してミュートを解除しようとしました。"""


class NameAlreadyExistsError(APIError):
    """同じURLにページがすでに存在します。"""


class NotBlockingError(APIError):
    """ブロックしていないユーザーです。"""


class NotFavoritedError(APIError):
    """お気に入り登録されていません。"""


class NotFollowingError(APIError):
    """ユーザーにフォローされていません。"""


class NotLikedError(APIError):
    """いいねをつけていないページです。"""


class NotMutingError(APIError):
    """対象となるユーザーをそもそもミュートしていません。"""


class NotReactedError(APIError):
    """リアクションしていません。"""


class NoFollowRequestError(APIError):
    """ユーザーからのフォローリクエストがありません。"""


class NoFreeSpaceError(APIError):
    """ドライブに空き容量がありません。"""


class NoPollError(APIError):
    """ """


class NoSuchAdError(APIError):
    """ """


class NoSuchAnnouncementError(APIError):
    """お知らせが存在しません。"""


class NoSuchAntennaError(APIError):
    """ """


class NoSuchAppError(APIError):
    """アプリが存在しません。"""


class NoSuchAvatarError(APIError):
    """アバター画像のファイルが存在しません。"""


class NoSuchBannerError(APIError):
    """バナー画像のファイルが存在しません。"""


class NoSuchChannelError(APIError):
    """指定されたチャンネルが存在しないか、アクセスが許可されていません。"""


class NoSuchClipError(APIError):
    """ """


class NoSuchEmojiError(APIError):
    """ """


class NoSuchFileError(APIError):
    """ファイルが存在しません。"""


class NoSuchFolderError(APIError):
    """フォルダーが存在しません。"""


class NoSuchGroupError(APIError):
    """ """


class NoSuchGroupMemberError(APIError):
    """ """


class NoSuchHashtagError(APIError):
    """ハッシュタグが存在しません。"""


class NoSuchInvitationError(APIError):
    """ """


class NoSuchListError(APIError):
    """ """


class NoSuchMessageError(APIError):
    """ """


class NoSuchNoteError(APIError):
    """指定されたノートが存在しないか、アクセスが許可されていません。"""


class NoSuchNotificationError(APIError):
    """通知が存在しません。"""


class NoSuchObjectError(APIError):
    """ """


class NoSuchPageError(APIError):
    """ページが存在しません。"""


class NoSuchParentFolderError(APIError):
    """親フォルダーが存在しません。"""


class NoSuchPostError(APIError):
    """ """


class NoSuchRenoteTargetError(APIError):
    """Renoteに指定されたノートが存在しないか、アクセスが許可されていません。"""


class NoSuchReplyTargetError(APIError):
    """返信先に指定されたノートが存在しないか、アクセスが許可されていません。"""


class NoSuchSessionError(APIError):
    """セッションが存在しません。"""


class NoSuchUserError(APIError):
    """ユーザーが存在しません。"""


class NoSuchUserGroupError(APIError):
    """ """


class NoSuchUserListError(APIError):
    """ """


class NoSuchWebhookError(APIError):
    """Webhookが存在しません。"""


class PendingSessionError(APIError):
    """ """


class PermissionDeniedError(APIError):
    """与えられたクレデンシャルには必要なパーミッションがありません。"""


class PinLimitExceededError(APIError):
    """これ以上ピン留めできません。"""


class RateLimitExceededError(APIError):
    """レートリミットによる制限のため一時的に利用できません。"""


class ReactionsNotPublicError(APIError):
    """リアクションが公開されていません。"""


class RecipientIsYourselfError(APIError):
    """ """


class StlDisabledError(APIError):
    """ソーシャルタイムラインが無効になっています。"""


class YourAccountSuspendedError(APIError):
    """アカウントが凍結されているため利用できません。"""


class YourPageError(APIError):
    """自身のページにいいねをつけようとしました。"""


class YourPostError(APIError):
    """ """


class YouAreOwnerError(APIError):
    """ """


class YouHaveBeenBlockedError(APIError):
    """ブロックされているユーザーのノートにリアクションは行えません。"""



"""
=======================
        WARNING
=======================
 This file is automatically generated by compiler/endpoints.py.
 If this file is modified and then auto-generated, the changes will be lost.
"""

from typing import Literal

ENDPOINTS = Literal[
    '/api/admin/meta',
    '/api/admin/abuse-user-reports',
    '/api/admin/accounts/create',
    '/api/admin/accounts/delete',
    '/api/admin/ad/create',
    '/api/admin/ad/delete',
    '/api/admin/ad/list',
    '/api/admin/ad/update',
    '/api/admin/announcements/create',
    '/api/admin/announcements/delete',
    '/api/admin/announcements/list',
    '/api/admin/announcements/update',
    '/api/admin/delete-all-files-of-a-user',
    '/api/admin/drive/clean-remote-files',
    '/api/admin/drive/cleanup',
    '/api/admin/drive/files',
    '/api/admin/drive/show-file',
    '/api/admin/emoji/add-aliases-bulk',
    '/api/admin/emoji/add',
    '/api/admin/emoji/copy',
    '/api/admin/emoji/delete-bulk',
    '/api/admin/emoji/delete',
    '/api/admin/emoji/list-remote',
    '/api/admin/emoji/list',
    '/api/admin/emoji/remove-aliases-bulk',
    '/api/admin/emoji/set-aliases-bulk',
    '/api/admin/emoji/set-category-bulk',
    '/api/admin/emoji/update',
    '/api/admin/federation/delete-all-files',
    '/api/admin/federation/refresh-remote-instance-metadata',
    '/api/admin/federation/remove-all-following',
    '/api/admin/federation/update-instance',
    '/api/admin/get-index-stats',
    '/api/admin/get-table-stats',
    '/api/admin/get-user-ips',
    '/api/invite',
    '/api/admin/promo/create',
    '/api/admin/queue/clear',
    '/api/admin/queue/deliver-delayed',
    '/api/admin/queue/inbox-delayed',
    '/api/admin/queue/promote',
    '/api/admin/queue/stats',
    '/api/admin/relays/add',
    '/api/admin/relays/list',
    '/api/admin/relays/remove',
    '/api/admin/reset-password',
    '/api/admin/resolve-abuse-user-report',
    '/api/admin/send-email',
    '/api/admin/server-info',
    '/api/admin/show-moderation-logs',
    '/api/admin/show-user',
    '/api/admin/show-users',
    '/api/admin/suspend-user',
    '/api/admin/unsuspend-user',
    '/api/admin/update-meta',
    '/api/admin/delete-account',
    '/api/admin/update-user-note',
    '/api/admin/roles/create',
    '/api/admin/roles/delete',
    '/api/admin/roles/list',
    '/api/admin/roles/show',
    '/api/admin/roles/update',
    '/api/admin/roles/assign',
    '/api/admin/roles/unassign',
    '/api/admin/roles/update-default-policies',
    '/api/admin/roles/users',
    '/api/announcements',
    '/api/antennas/create',
    '/api/antennas/delete',
    '/api/antennas/list',
    '/api/antennas/notes',
    '/api/antennas/show',
    '/api/antennas/update',
    '/api/ap/get',
    '/api/ap/show',
    '/api/app/create',
    '/api/app/show',
    '/api/auth/session/generate',
    '/api/auth/session/show',
    '/api/auth/session/userkey',
    '/api/blocking/create',
    '/api/blocking/delete',
    '/api/blocking/list',
    '/api/channels/create',
    '/api/channels/featured',
    '/api/channels/follow',
    '/api/channels/followed',
    '/api/channels/owned',
    '/api/channels/show',
    '/api/channels/timeline',
    '/api/channels/unfollow',
    '/api/channels/update',
    '/api/charts/active-users',
    '/api/charts/ap-request',
    '/api/charts/drive',
    '/api/charts/federation',
    '/api/charts/instance',
    '/api/charts/notes',
    '/api/charts/user/drive',
    '/api/charts/user/following',
    '/api/charts/user/notes',
    '/api/charts/user/pv',
    '/api/charts/user/reactions',
    '/api/charts/users',
    '/api/clips/add-note',
    '/api/clips/remove-note',
    '/api/clips/create',
    '/api/clips/delete',
    '/api/clips/list',
    '/api/clips/notes',
    '/api/clips/show',
    '/api/clips/update',
    '/api/clips/favorite',
    '/api/clips/unfavorite',
    '/api/clips/my-favorites',
    '/api/drive',
    '/api/drive/files',
    '/api/drive/files/attached-notes',
    '/api/drive/files/check-existence',
    '/api/drive/files/create',
    '/api/drive/files/delete',
    '/api/drive/files/find-by-hash',
    '/api/drive/files/find',
    '/api/drive/files/show',
    '/api/drive/files/update',
    '/api/drive/files/upload-from-url',
    '/api/drive/folders',
    '/api/drive/folders/create',
    '/api/drive/folders/delete',
    '/api/drive/folders/find',
    '/api/drive/folders/show',
    '/api/drive/folders/update',
    '/api/drive/stream',
    '/api/email-address/available',
    '/api/endpoint',
    '/api/endpoints',
    '/api/federation/followers',
    '/api/federation/following',
    '/api/federation/instances',
    '/api/federation/show-instance',
    '/api/federation/update-remote-user',
    '/api/federation/users',
    '/api/federation/stats',
    '/api/following/create',
    '/api/following/delete',
    '/api/following/invalidate',
    '/api/following/requests/accept',
    '/api/following/requests/cancel',
    '/api/following/requests/list',
    '/api/following/requests/reject',
    '/api/gallery/featured',
    '/api/gallery/popular',
    '/api/gallery/posts',
    '/api/gallery/posts/create',
    '/api/gallery/posts/delete',
    '/api/gallery/posts/like',
    '/api/gallery/posts/show',
    '/api/gallery/posts/unlike',
    '/api/gallery/posts/update',
    '/api/get-online-users-count',
    '/api/hashtags/list',
    '/api/hashtags/search',
    '/api/hashtags/show',
    '/api/hashtags/trend',
    '/api/hashtags/users',
    '/api/i',
    '/api/i/claim-achievement',
    '/api/i/favorites',
    '/api/i/gallery/likes',
    '/api/i/gallery/posts',
    '/api/i/get-word-muted-notes-count',
    '/api/i/notifications',
    '/api/i/page-likes',
    '/api/i/pages',
    '/api/i/pin',
    '/api/i/read-all-unread-notes',
    '/api/i/read-announcement',
    '/api/i/unpin',
    '/api/i/update',
    '/api/i/webhooks/create',
    '/api/i/webhooks/list',
    '/api/i/webhooks/show',
    '/api/i/webhooks/update',
    '/api/i/webhooks/delete',
    '/api/meta',
    '/api/emojis',
    '/api/emoji',
    '/api/mute/create',
    '/api/mute/delete',
    '/api/mute/list',
    '/api/renote-mute/create',
    '/api/renote-mute/delete',
    '/api/renote-mute/list',
    '/api/my/apps',
    '/api/notes',
    '/api/notes/children',
    '/api/notes/clips',
    '/api/notes/conversation',
    '/api/notes/create',
    '/api/notes/delete',
    '/api/notes/favorites/create',
    '/api/notes/favorites/delete',
    '/api/notes/featured',
    '/api/notes/global-timeline',
    '/api/notes/hybrid-timeline',
    '/api/notes/local-timeline',
    '/api/notes/mentions',
    '/api/notes/polls/recommendation',
    '/api/notes/polls/vote',
    '/api/notes/reactions',
    '/api/notes/reactions/create',
    '/api/notes/reactions/delete',
    '/api/notes/renotes',
    '/api/notes/replies',
    '/api/notes/search-by-tag',
    '/api/notes/search',
    '/api/notes/show',
    '/api/notes/state',
    '/api/notes/thread-muting/create',
    '/api/notes/thread-muting/delete',
    '/api/notes/timeline',
    '/api/notes/translate',
    '/api/notes/unrenote',
    '/api/notes/user-list-timeline',
    '/api/notifications/create',
    '/api/notifications/mark-all-as-read',
    '/api/notifications/read',
    '/api/pages/create',
    '/api/pages/delete',
    '/api/pages/featured',
    '/api/pages/like',
    '/api/pages/show',
    '/api/pages/unlike',
    '/api/pages/update',
    '/api/flash/create',
    '/api/flash/delete',
    '/api/flash/featured',
    '/api/flash/like',
    '/api/flash/show',
    '/api/flash/unlike',
    '/api/flash/update',
    '/api/flash/my',
    '/api/flash/my-likes',
    '/api/ping',
    '/api/pinned-users',
    '/api/promo/read',
    '/api/roles/list',
    '/api/roles/show',
    '/api/roles/users',
    '/api/request-reset-password',
    '/api/reset-db',
    '/api/reset-password',
    '/api/server-info',
    '/api/stats',
    '/api/sw/show-registration',
    '/api/sw/update-registration',
    '/api/sw/register',
    '/api/sw/unregister',
    '/api/test',
    '/api/username/available',
    '/api/users',
    '/api/users/clips',
    '/api/users/followers',
    '/api/users/following',
    '/api/users/gallery/posts',
    '/api/users/get-frequently-replied-users',
    '/api/users/lists/create',
    '/api/users/lists/delete',
    '/api/users/lists/list',
    '/api/users/lists/pull',
    '/api/users/lists/push',
    '/api/users/lists/show',
    '/api/users/lists/update',
    '/api/users/notes',
    '/api/users/pages',
    '/api/users/reactions',
    '/api/users/recommendation',
    '/api/users/relation',
    '/api/users/report-abuse',
    '/api/users/search-by-username-and-host',
    '/api/users/search',
    '/api/users/show',
    '/api/users/stats',
    '/api/users/achievements',
    '/api/fetch-rss',
    '/api/retention',
    '/api/admin/delete-logs',
    '/api/admin/emoji/remove',
    '/api/admin/invite',
    '/api/admin/logs',
    '/api/admin/moderators/add',
    '/api/admin/moderators/remove',
    '/api/admin/queue/jobs',
    '/api/admin/remove-abuse-user-report',
    '/api/admin/resync-chart',
    '/api/admin/set-premium',
    '/api/admin/silence-user',
    '/api/admin/unset-premium',
    '/api/admin/unsilence-user',
    '/api/admin/unverify-user',
    '/api/admin/update-remote-user',
    '/api/admin/vacuum',
    '/api/admin/verify-user',
    '/api/channels/pin-note',
    '/api/charts/hashtag',
    '/api/charts/network',
    '/api/games/reversi/games',
    '/api/games/reversi/games/show',
    '/api/games/reversi/games/surrender',
    '/api/games/reversi/invitations',
    '/api/games/reversi/match',
    '/api/games/reversi/match/cancel',
    '/api/i/read-all-messaging-messages',
    '/api/i/registry/get-all',
    '/api/i/registry/get-detail',
    '/api/i/registry/get',
    '/api/i/registry/keys-with-type',
    '/api/i/registry/keys',
    '/api/i/registry/remove',
    '/api/i/registry/scopes',
    '/api/i/registry/set',
    '/api/i/user-group-invites',
    '/api/messaging/history',
    '/api/messaging/messages',
    '/api/messaging/messages/create',
    '/api/messaging/messages/delete',
    '/api/messaging/messages/read',
    '/api/notes/watching/create',
    '/api/notes/watching/delete',
    '/api/room/show',
    '/api/room/update',
    '/api/users/groups/create',
    '/api/users/groups/delete',
    '/api/users/groups/invitations/accept',
    '/api/users/groups/invitations/reject',
    '/api/users/groups/invite',
    '/api/users/groups/joined',
    '/api/users/groups/owned',
    '/api/users/groups/pull',
    '/api/users/groups/show',
    '/api/users/groups/transfer',
    '/api/users/groups/update',
    '/api/version',
    '/api/channels/favorite',
    '/api/channels/unfavorite',
    '/api/channels/my-favorites',
    '/api/channels/search',
    '/api/roles/notes',
    '/api/users/update-memo',
    '/api/admin/emoji/set-license-bulk',
    '/api/admin/invite/create',
    '/api/admin/invite/list',
    '/api/invite/create',
    '/api/invite/delete',
    '/api/invite/list',
    '/api/invite/limit',
    '/api/users/lists/favorite',
    '/api/users/lists/unfavorite',
    '/api/users/lists/create-from-public',
    '/api/users/flashs'
]



from dataclasses import dataclass
from typing import Literal, Self, TypedDict


@dataclass
class CacheConfigData:
    maxsize: int = 1024
    ttl: int = 360


class CacheConfig:
    def __init__(self, options: CacheConfigData) -> None:
        self.maxsize: int = options.maxsize
        self.ttl: int = options.ttl


IMisskeyDistribution = Literal["ayuskey", "m544", "areionskey", "official"]
IMisskeyVersions = Literal[13, 12, 11]


class ILimits(TypedDict, total=False):
    channel_name: int
    channel_description: int


class IFeatures(TypedDict, total=False):
    chat: bool


class Limits:
    def __init__(self, limits: ILimits | None = None) -> None:
        limits = limits or {}
        self.channel_name: int = limits.get("channel_name", 128)
        self.channel_description: int = limits.get("channel_description", 2048)

    def from_dict(self, limits: ILimits):
        self.channel_name = limits.get("channel_name") or self.channel_description
        self.channel_description = limits.get("channel_description") or self.channel_description
        return self


class Features:
    def __init__(self, features: IFeatures | None = None) -> None:
        features = features or {}
        self.chat = features.get("chat", False)

    def from_dict(self, features: IFeatures):
        self.chat = features.get("chat") or self.chat
        return self


class Config:
    def __init__(
        self,
        *,
        host: str = "",
        is_ssl: bool = True,
        distro: IMisskeyDistribution = "official",
        is_ayuskey: bool = False,
        use_version: IMisskeyVersions = 12,
        cache: CacheConfigData | None = None,
        use_version_autodetect: bool = True,
        features: IFeatures | None = None,
        limits: ILimits | None = None,
    ) -> None:
        self.account_id: str = ""
        self.distro: IMisskeyDistribution = distro
        self.is_ssl: bool = is_ssl
        self.host: str = host
        self.is_ayuskey: bool = is_ayuskey
        self.use_version: IMisskeyVersions = use_version
        self.cache: CacheConfig = CacheConfig(cache or CacheConfigData())
        self.use_version_autodetect: bool = use_version_autodetect
        self.features: Features = Features(features) if features else Features()
        self.limits: Limits = Limits(limits) if limits else Limits()

    def from_dict(
        self,
        *,
        host: str | None = None,
        is_ssl: bool | None = None,
        is_ayuskey: bool | None = None,
        use_version: IMisskeyVersions | None = None,
        cache: CacheConfigData | None = None,
        use_version_autodetect: bool | None = None,
        features: IFeatures | None = None,
        limits: ILimits | None = None,
        account_id: str | None = None,
    ) -> Self:
        self.host = host or self.host
        self.is_ssl = is_ssl if is_ssl is not None else self.is_ssl
        self.is_ayuskey = is_ayuskey or self.is_ayuskey
        self.use_version = use_version or self.use_version
        if cache:
            self.cache = CacheConfig(cache)
        self.use_version_autodetect = use_version_autodetect or self.use_version_autodetect
        self.features = self.features.from_dict(features) if features else self.features
        self.limits = self.limits.from_dict(limits) if limits else self.limits
        self.account_id = account_id or self.account_id
        return self


config = Config()



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, Literal, overload

from mipac.abstract.action import AbstractAction
from mipac.errors.base import ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.announcement import Announcement
from mipac.models.lite.meta import LiteMeta
from mipac.models.meta import Meta
from mipac.types.announcement import IAnnouncement
from mipac.types.meta import ILiteMeta, IMeta
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager) -> None:
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @overload
    async def get_meta(self, detail: Literal[False] = ...) -> LiteMeta:
        ...

    @overload
    async def get_meta(self, detail: Literal[True] = ...) -> Meta:
        ...

    async def get_meta(self, detail: bool = False):
        params = {
            "route": Route("POST", "/api/meta"),
            "json": {"detail": detail},
            "auth": True,
            "lower": True,
        }
        if detail is True:
            meta: IMeta = await self.__session.request(**params)
            return Meta(meta, client=self.__client)
        lite_meta: ILiteMeta = await self.__session.request(**params)
        return LiteMeta(lite_meta, client=self.__client)

    async def get_announcements(
        self,
        limit: int = 10,
        with_unreads: bool = False,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        get_all: bool = False,
    ) -> AsyncGenerator[Announcement, None]:
        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")
        if get_all:
            limit = 100

        body = {
            "limit": limit,
            "withUnreads": with_unreads,
            "sinceId": since_id,
            "untilId": until_id,
        }

        pagination = Pagination[IAnnouncement](
            self.__session, Route("POST", "/api/announcements"), json=body
        )

        while True:
            res_announcements = await pagination.next()
            for announcement in res_announcements:
                yield Announcement(announcement, client=self.__client)

            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.user import UserActions
from mipac.http import HTTPClient
from mipac.manager.blocking import BlockingManager
from mipac.manager.follow import FollowManager
from mipac.manager.mute import MuteManager

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.models.lite.user import LiteUser


__all__ = ("UserManager",)


class UserManager(AbstractManager):
    def __init__(
        self, user: LiteUser | None = None, *, session: HTTPClient, client: ClientManager
    ):
        user_id: str | None = user.id if user else None
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__user: LiteUser | None = user
        self.follow: FollowManager = FollowManager(user_id=user_id, session=session, client=client)
        self.mute: MuteManager = MuteManager(user_id=user_id, session=session, client=client)
        self.block = BlockingManager(user_id=user_id, session=session, client=client)

    @property
    def action(self) -> UserActions:
        """ユーザーに対するアクション

        Returns
        -------
        UserActions
            ユーザーに対するアクションを行うクラス
        """
        return UserActions(session=self.__session, client=self.__client, user=self.__user)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.federation import FederationActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class FederationManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session = session
        self.__client = client

    @property
    def action(self) -> FederationActions:
        return FederationActions(session=self.__session, client=self.__client)



from typing import Any, Generic, Literal, NotRequired, Optional, TypedDict, TypeVar

from mipac.types.drive import IDriveFile
from mipac.types.emoji import ICustomEmojiLite
from mipac.types.poll import IPoll
from mipac.types.reaction import IReactionAcceptance
from mipac.types.user import ILiteUser

T = TypeVar("T")

INoteVisibility = Literal["public", "home", "followers", "specified"]


class INoteState(TypedDict):
    is_favorited: bool
    is_watching: bool
    is_muted_thread: NotRequired[bool]


class INoteUpdated(TypedDict, Generic[T]):
    type: Literal["noteUpdated"]
    body: T


class INoteUpdatedDeleteBody(TypedDict):
    deleted_at: str


class INoteUpdatedDelete(TypedDict):
    id: str
    type: Literal["deleted"]
    body: INoteUpdatedDeleteBody


class INoteUpdatedReactionBody(TypedDict):
    reaction: str
    emoji: ICustomEmojiLite
    user_id: str


class INoteUpdatedReaction(TypedDict):
    id: str
    type: Literal["reacted", "unreacted"]
    body: INoteUpdatedReactionBody


class GeoPayload(TypedDict):
    """
    衛星情報
    """

    coordinates: Optional[list[Any]]
    altitude: int | None
    accuracy: int | None
    altitude_accuracy: int | None
    heading: int | None
    speed: int | None


class IPartialNote(TypedDict):
    created_at: str
    cw: str | None
    file_ids: list[str]
    files: list[IDriveFile]
    id: str
    reaction_acceptance: NotRequired[IReactionAcceptance]  # v13 only
    reaction_emojis: NotRequired[dict[str, str]]  # v13 only
    renote_id: str | None
    renote_count: int
    reactions: dict[str, int]
    replies_count: int
    reply_id: str | None
    text: str | None
    user: ILiteUser
    user_id: str
    visibility: INoteVisibility
    tags: NotRequired[list[str]]  # タグがついてないとbodyに存在しない


class INote(IPartialNote, total=False):
    """
    note object
    """

    renote: "INote"
    reply: "INote"
    visible_user_ids: list[str]
    local_only: bool
    my_reaction: str
    uri: str
    url: str
    is_hidden: bool
    poll: IPoll
    emojis: list[ICustomEmojiLite]


class ICreatedNote(TypedDict):
    """
    created note
    """

    created_note: INote


class INoteReaction(TypedDict):
    id: str
    created_at: str
    user: ILiteUser
    type: str


class INoteTranslateResult(TypedDict):
    sourceLang: str
    text: str



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.errors.base import ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.announcement import Announcement, AnnouncementSystem
from mipac.types.announcement import IAnnouncement, IAnnouncementSystem
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class AdminAnnouncementClientActions(AbstractAction):
    def __init__(
        self,
        announce_id: str | None = None,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        self.__announce_id = announce_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def delete(self, announce_id: str | None = None) -> bool:
        announce_id = announce_id or self.__announce_id
        res: bool = await self.__session.request(
            Route("POST", "/api/admin/announcements/delete"),
            json={"id": announce_id},
            auth=True,
        )
        return res

    async def update(
        self,
        title: str,
        text: str,
        image_url: str | None = None,
        *,
        announce_id: str | None = None,
    ):
        announce_id = announce_id or self.__announce_id
        body = {
            "id": announce_id,
            "title": title,
            "text": text,
            "imageUrl": image_url,
        }
        res: bool = await self.__session.request(
            Route("POST", "/api/admin/announcements/update"),
            json=body,
            auth=True,
            remove_none=False,
        )
        return res


class AdminAnnouncementActions(AdminAnnouncementClientActions):
    def __init__(
        self,
        announce_id: str | None = None,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        super().__init__(announce_id=announce_id, session=session, client=client)

    async def create(self, title: str, text: str, image_url: str | None = None) -> Announcement:
        body = {"title": title, "text": text, "imageUrl": image_url}
        created_announcement: IAnnouncement = await self.__session.request(
            Route("POST", "/api/admin/announcements/create"),
            json=body,
            auth=True,
            lower=True,
            remove_none=False,
        )
        return Announcement(created_announcement, client=self.__client)

    async def gets(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        get_all: bool = False,
    ) -> AsyncGenerator[AnnouncementSystem, None]:
        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")
        if get_all:
            limit = 100

        body = {
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
        }

        pagination = Pagination[IAnnouncementSystem](
            self.__session, Route("POST", "/api/admin/announcements/list"), json=body
        )

        while True:
            res_annonuncement_systems = await pagination.next()
            for res_announcement_system in res_annonuncement_systems:
                yield AnnouncementSystem(res_announcement_system, client=self.__client)

            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.errors.base import NotSupportVersion, NotSupportVersionText
from mipac.http import HTTPClient, Route
from mipac.models.follow import FollowRequest
from mipac.types.achievement import IT_ACHIEVEMENT_NAME
from mipac.types.follow import IFollowRequest

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class MyActions(AbstractAction):
    def __init__(self, session: HTTPClient, client: ClientManager):
        self.__session = session
        self.__client = client

    async def fetch_follow_requests(self) -> list[FollowRequest]:
        res: list[IFollowRequest] = await self.__session.request(
            Route("POST", "/api/following/requests/list"),
            auth=True,
            lower=True,
        )
        return [FollowRequest(i, client=self.__client) for i in res]

    async def get_claim_achievement(self, name: IT_ACHIEVEMENT_NAME) -> bool:
        """指定した名前の実績を解除します

        Parameters
        ----------
        name : 実績名
            解除したい実績の名前

        Returns
        -------
        bool
            成功したか否か

        Raises
        ------
        NotSupportVersion
            実績機能が存在しないサーバーを使用している
        """
        if self.__client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)
        res: bool = await self.__session.request(
            Route("POST", "/api/i/claim-achievement"), auth=True, json={"name": name}, lower=True
        )
        return res



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.user import UserDetailed
from mipac.types.mute import IMuteUser

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class MuteUser(AbstractModel):
    def __init__(self, data: IMuteUser, *, client: ClientManager) -> None:
        self.__data: IMuteUser = data
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__data["id"]

    @property
    def created_at(self) -> str:
        return self.__data["created_at"]

    @property
    def mutee_id(self) -> str:
        return self.__data["mutee_id"]

    @property
    def mutee(self) -> UserDetailed:
        return UserDetailed(self.__data["mutee"], client=self.__client)

    def __eq__(self, __value: MuteUser) -> bool:
        return isinstance(__value, IMuteUser) and self.id == __value.id

    def __ne__(self, __value: MuteUser) -> bool:
        return not self.__eq__(__value)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.follow import FollowActions, FollowRequestActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.client import ClientManager

__all__ = ("FollowManager", "FollowRequestManager")


class FollowManager(AbstractManager):
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__user_id: str | None = user_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.request: FollowRequestManager = FollowRequestManager(
            user_id=user_id, session=session, client=client
        )

    @property
    def action(self) -> FollowActions:
        return FollowActions(
            user_id=self.__user_id,
            session=self.__session,
            client=self.__client,
        )


class FollowRequestManager(AbstractManager):
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__user_id: str | None = user_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> FollowRequestActions:
        return FollowRequestActions(
            user_id=self.__user_id,
            session=self.__session,
            client=self.__client,
        )



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Generic, TypeVar

from mipac.abstract.model import AbstractModel
from mipac.types.channel import IChannelLite
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager import ClientManager
    from mipac.manager.channel import ChannelManager


T = TypeVar("T", bound=IChannelLite)


class ChannelLite(Generic[T], AbstractModel):
    def __init__(self, channel: T, *, client: ClientManager) -> None:
        self._channel: T = channel
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self._channel["id"]

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self._channel["created_at"])

    @property
    def last_noted_at(self) -> datetime | None:
        last_noted_at = self._channel.get("last_noted_at")
        return str_to_datetime(last_noted_at) if last_noted_at else None

    @property
    def name(self) -> str:
        return self._channel["name"]

    @property
    def description(self) -> str | None:
        return self._channel["description"]

    @property
    def user_id(self) -> str:
        return self._channel["user_id"]

    @property
    def banner_url(self) -> str | None:
        return self._channel["banner_url"]

    @property
    def users_count(self) -> int:
        return self._channel["users_count"]

    @property
    def notes_count(self) -> int:
        return self._channel["notes_count"]

    @property
    def pinned_note_ids(self) -> list:
        return self._channel.get("pinned_note_ids", [])

    @property
    def api(self) -> ChannelManager:
        return self.__client._create_channel_instance(channel_id=self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, ChannelLite) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, Literal

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.invite import InviteCode, PartialInviteCode
from mipac.types.invite import IInviteCode, IPartialInviteCode
from mipac.util import remove_dict_empty
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.client import ClientManager


class AdminInviteActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def create_invite(
        self, count: int = 1, expires_at: str | None = None
    ) -> list[PartialInviteCode]:
        raw_codes: list[IPartialInviteCode] = await self.__session.request(
            route=Route(method="POST", path="/api/admin/invite/create"),
            json={"count": count, "expiresAt": expires_at},
            auth=True,
        )
        return [PartialInviteCode(raw_code, client=self.__client) for raw_code in raw_codes]

    async def get_invite_list(
        self,
        limit: int = 30,
        offset: int = 0,
        type: Literal["unused", "used", "expired", "all"] = "all",
        sort: Literal["+createdAt", "-createdAt", "+usedAt", "-usedAt"] = "+createdAt",
        *,
        get_all: bool = False,
    ) -> AsyncGenerator[InviteCode, None]:
        pagination = Pagination[IInviteCode](
            http_client=self.__session,
            route=Route("POST", "/api/admin/invite/list"),
            json=remove_dict_empty({"limit": limit, "offset": offset, "type": type, "sort": sort}),
            pagination_type="count",
            auth=True,
        )
        while True:
            raw_invite_codes = await pagination.next()
            for raw_invite_code in raw_invite_codes:
                yield InviteCode(raw_invite_code, client=self.__client)

            if pagination.is_final or get_all is False:
                break



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.admins.roles import AdminRoleActions, AdminRoleModelActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.client import ClientManager


class AdminRolesModelManager(AbstractManager):
    def __init__(self, role_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__role_id = role_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> AdminRoleModelActions:
        return AdminRoleModelActions(
            role_id=self.__role_id, session=self.__session, client=self.__client
        )


class AdminRolesManager:
    def __init__(self, role_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__role_id = role_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> AdminRoleActions:
        return AdminRoleActions(
            role_id=self.__role_id, session=self.__session, client=self.__client
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.types.instance import IFederationInstance

if TYPE_CHECKING:
    from mipac.client import ClientManager


class FederationInstance(AbstractModel):
    def __init__(self, instance: IFederationInstance, *, client: ClientManager) -> None:
        self.__instance: IFederationInstance = instance
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__instance["id"]

    @property
    def host(self) -> str:
        return self.__instance["host"]

    @property
    def users_count(self) -> int:
        return self.__instance["users_count"]

    @property
    def notes_count(self) -> int:
        return self.__instance["notes_count"]

    @property
    def following_count(self) -> int:
        return self.__instance["following_count"]

    @property
    def followers_count(self) -> int:
        return self.__instance["followers_count"]

    @property
    def is_not_responding(self) -> bool:
        return self.__instance["is_not_responding"]

    @property
    def is_suspended(self) -> bool:
        return self.__instance["is_suspended"]

    @property
    def is_blocked(self) -> bool:
        return self.__instance["is_blocked"]

    @property
    def software_name(self) -> str:
        return self.__instance["software_name"]

    @property
    def software_version(self) -> str:
        return self.__instance["software_version"]

    @property
    def open_registrations(self) -> bool:
        return self.__instance["open_registrations"]

    @property
    def name(self) -> str:
        return self.__instance["name"]

    @property
    def description(self) -> str:
        return self.__instance["description"]

    @property
    def maintainer_name(self) -> str:
        return self.__instance["maintainer_name"]

    @property
    def maintainer_email(self) -> str:
        return self.__instance["maintainer_email"]

    @property
    def icon_url(self) -> str:
        return self.__instance["icon_url"]

    @property
    def favicon_url(self) -> str:
        return self.__instance["favicon_url"]

    @property
    def theme_color(self) -> str:
        return self.__instance["theme_color"]

    @property
    def info_updated_at(self) -> str:
        return self.__instance["info_updated_at"]

    @property
    def caught_at(self) -> str | None:
        return self.__instance.get("caught_at")

    @property
    def first_retrieved_at(self) -> str | None:
        return self.__instance.get("first_retrieved_at")

    @property
    def latest_request_sent_at(self) -> str | None:
        return self.__instance.get("latest_request_sent_at")

    @property
    def last_communicated_at(self) -> str | None:
        return self.__instance.get("last_communicated_at")

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, FederationInstance) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from typing import NotRequired, TypedDict

from mipac.types.ads import IAds
from mipac.types.emoji import ICustomEmoji


class ICPU(TypedDict):
    cores: int
    model: str


class IPolicies(TypedDict):
    gtl_available: bool
    ltl_available: bool
    can_public_note: bool
    can_invite: bool
    can_manage_custom_emojis: bool
    can_hide_ads: bool
    drive_capacity_mb: int
    pin_limit: int
    antenna_limit: int
    word_mute_limit: int
    webhook_limit: int
    clip_limit: int
    note_each_clips_limit: int
    user_list_limit: int
    user_each_user_lists_limit: int
    rate_limit_factor: int


class IMetaAnnouncement(TypedDict):
    image: str | None
    text: str
    title: str


class IV12Features(TypedDict, total=False):
    email_required_for_signup: bool
    miauth: bool


class IV11Features(TypedDict, total=False):
    global_time_line: bool
    local_time_line: bool


class IFeatures(IV12Features, IV11Features):
    registration: bool
    elasticsearch: bool
    hcaptcha: bool
    recaptcha: bool
    object_storage: bool
    twitter: bool
    github: bool
    discord: bool
    service_worker: bool


class IV12AdminMeta(TypedDict, total=False):
    hcaptcha_secret_key: str
    sensitive_media_detection: str
    sensitive_media_detection_sensitivity: str
    set_sensitive_flag_automatically: bool
    enable_sensitive_media_detection_for_videos: bool
    proxy_account_id: str
    summary_proxy: str
    enable_ip_logging: bool
    enable_active_email_validation: bool


class ISharedAdminMeta(TypedDict, total=False):
    drive_capacity_per_local_user_mb: int
    drive_capacity_per_remote_user_mb: int
    hidden_tags: list[str]
    blocked_hosts: list[str]
    recaptcha_secret_key: str
    twitter_consumer_key: str
    twitter_consumer_secret: str
    github_client_id: str
    github_client_secret: str
    discord_client_id: str
    discord_client_secret: str
    email: str
    smtp_secure: bool
    smtp_host: str
    smtp_port: int
    smtp_user: str
    smtp_pass: str
    sw_private_key: str
    use_object_storage: bool
    object_storage_base_url: str
    object_storage_bucket: str
    object_storage_prefix: str
    object_storage_endpoint: str
    object_storage_region: str
    object_storage_port: int
    object_storage_access_key: str
    object_storage_secret_key: str
    object_storage_use_ssl: bool
    object_storage_use_proxy: bool
    object_storage_set_public_read: bool
    pinned_users: list[str]


class ILiteV12Meta(TypedDict, total=False):
    background_image_url: str
    default_dark_theme: str
    default_light_theme: str
    logo_image_url: str
    theme_color: str


class IMetaCommonV12(TypedDict, total=False):
    ads: list[IAds]
    translator_available: bool  # v12 only
    email_required_for_signup: bool
    mascot_image_url: str


class ICommonV11(TypedDict, total=False):
    object_storage_s3_force_path_style: bool


class ILiteV11Meta(ICommonV11, total=False):
    announcements: list[IMetaAnnouncement]
    cpu: ICPU
    disable_local_timeline: bool
    disable_global_timeline: bool
    enable_emoji_reaction: bool
    machine: str
    node: str
    os: str
    proxy_account: str
    proxy_remote_files: bool
    psql: str
    redis: str
    secure: bool
    summaly_proxy: str
    tos_text_url: str
    turnstile_secret_key: str


class IMetaCommon(IMetaCommonV12, ICommonV11):
    cache_remote_files: bool
    enable_hcaptch: bool
    hcaptcha_site_key: str | None
    enable_recaptcha: bool
    recaptcha_site_key: str
    sw_publickey: str | None
    banner_url: str | None
    error_image_url: str | None
    icon_url: str | None
    max_note_text_length: int
    enable_email: bool
    enable_twitter_integration: bool
    enable_github_integration: bool
    enable_discord_integration: bool
    enable_service_worker: bool
    proxy_account_name: str | None
    use_star_for_reaction_fallback: bool
    emojis: NotRequired[list[ICustomEmoji]]


class ILiteMeta(IMetaCommon, ILiteV12Meta, ILiteV11Meta, ISharedAdminMeta):
    description: str | None
    disable_registration: bool
    feedback_url: str
    maintainer_email: str | None
    maintainer_name: str | None
    name: str | None
    repository_url: str
    tos_url: str  # 厳密にはv11とv12で異なるが、スネークケースに置き換える都合上問題ない
    uri: str
    version: str


class IV12Meta(TypedDict, total=False):
    pinned_clip_id: str
    pinned_pages: list[str]
    policies: IPolicies
    require_setup: bool


class IMeta(ILiteMeta, IV12Meta):
    features: IFeatures


class IAdminMeta(ISharedAdminMeta, IV12AdminMeta, IMetaCommon):
    pass


class IUpdateMetaBody(TypedDict, total=False):
    announcements: list
    disable_registration: bool
    disable_local_timeline: bool
    disable_global_timeline: bool
    enable_emoji_reaction: bool
    use_star_for_reaction_fallback: bool
    pinned_users: list[str]
    hidden_tags: list[str]
    blocked_hosts: list[str]
    mascot_image_url: str | None
    banner_url: str | None
    error_image_url: str | None
    icon_url: str | None
    name: str | None
    description: str | None
    max_note_text_length: int
    local_drive_capacity_mb: int
    remote_drive_capacity_mb: int
    cache_remote_files: bool
    proxy_remote_files: bool
    enable_recaptcha: bool
    recaptcha_site_key: str
    recaptcha_secret_key: str
    enable_turnstile: bool
    turnstile_site_key: str
    turnstile_secret_key: str
    proxy_account_id: str | None
    proxy_account: str
    maintainer_name: str | None
    maintainer_email: str | None
    langs: str
    summaly_proxy: str | None
    enable_twitter_integration: bool
    twitter_consumer_key: str | None
    twitter_consumer_secret: str | None
    enable_github_integration: bool
    github_client_id: str | None
    github_client_secret: str | None
    enable_discord_integration: bool
    discord_client_id: str | None
    discord_client_secret: str | None
    enable_email: bool
    email: str
    smtp_secure: bool
    smtp_host: str | None
    smtp_port: int | None
    smtp_user: str | None
    smtp_pass: str | None
    enable_service_worker: bool
    sw_public_key: str
    sw_private_key: str
    tos_url: str | None
    tos_text_url: str
    repository_url: str
    feedback_url: str
    use_object_storage: bool
    object_storage_base_url: str | None
    object_storage_bucket: str | None
    object_storage_prefix: str | None
    object_storage_endpoint: str | None
    object_storage_region: str | None
    object_storage_port: int | None
    object_storage_access_key: str | None
    object_storage_secret_key: str | None
    object_storage_use_s_s_l: bool
    object_storage_use_proxy: bool
    object_storage_set_public_read: bool
    object_storage_s3_force_path_style: bool
    server_rules: NotRequired[list[str]]  # v13.11.3以降のバージョンから追加。その場合は使わないとエラー出るかも



# noqa: F403, F401

from .admins import *
from .chart import *
from .chat import *
from .client import *
from .drive import *
from .favorite import *
from .follow import *
from .mute import *
from .my import *
from .note import *
from .page import *
from .poll import *
from .reaction import *
from .user import *



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.actions.admins.user import AdminUserActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.client import ClientManager


class AdminUserManager:
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__user_id = user_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> AdminUserActions:
        return AdminUserActions(
            user_id=self.__user_id, session=self.__session, client=self.__client
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.config import config
from mipac.types.emoji import ICustomEmojiLite

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class PartialCustomEmoji(AbstractModel):
    def __init__(self, emoji: ICustomEmojiLite, *, client: ClientManager) -> None:
        self.__emoji = emoji
        self.__client: ClientManager = client

    @property
    def name(self) -> str:
        return self.__emoji["name"]

    @property
    def url(self) -> str | None:
        if config.use_version == 13:
            protocol = "https" if config.is_ssl else "http"
            url = f"{protocol}://{config.host}/emoji/{self.name}.webp"
            return url
        return self.__emoji.get("url")



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.errors.base import ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.clip import Clip
from mipac.models.note import Note
from mipac.types.clip import IClip
from mipac.types.note import INote
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientClipActions(AbstractAction):
    def __init__(self, *, clip_id: str | None = None, session: HTTPClient, client: ClientManager):
        self._clip_id = clip_id
        self._session = session
        self._client = client

    async def get_notes(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        get_all: bool = False,
        clip_id: str | None = None,
    ) -> AsyncGenerator[Note, None]:
        """Get notes from a clip
        Parameters
        ----------
        clip_id : str | None, optional, by default None
            The clip id
        limit : int, optional, by default 10
            The number of notes to get
        since_id : str | None, optional, by default None
            The note id to get notes after
        until_id : str | None, optional, by default None
            The note id to get notes before
        get_all : bool, optional, by default False
            Whether to get all notes

        Yields
        ------
        AsyncGenerator[Note, None]
            The notes
        """

        clip_id = self._clip_id or clip_id

        if clip_id is None:
            raise ParameterError("clip_id is required")

        if limit > 100:
            raise ParameterError("limit must be less than 100")

        if get_all:
            limit = 100

        body = {"clipId": clip_id, "limit": limit, "sinceId": since_id, "untilId": until_id}

        pagination = Pagination[INote](
            self._session, Route("POST", "/api/clips/notes"), json=body, auth=True
        )

        while True:
            clips = await pagination.next()
            for raw_clip in clips:
                yield Note(raw_clip, client=self._client)

            if get_all is False or pagination.is_final:
                break

    async def add_note(self, note_id: str, clip_id: str | None = None) -> bool:
        """Add a note to a clip

        Parameters
        ----------
        clip_id : str | None, optional, by default None
            The clip id
        note_id : str
            The note id

        Returns
        -------
        bool
            True if the note was added to the clip, False otherwise
        """
        clip_id = self._clip_id or clip_id

        if clip_id is None:
            raise ParameterError("clip_id is required")

        body = {"clipId": clip_id, "noteId": note_id}
        result: bool = await self._session.request(
            Route("POST", "/api/clips/add-note"), json=body, auth=True
        )
        return result

    async def remove_note(self, note_id: str, clip_id: str | None) -> bool:
        """Remove a note from a clip

        Parameters
        ----------
        clip_id : str | None, optional, by default None
            The clip id
        note_id : str
            The note id

        Returns
        -------
        bool
            True if the note was removed from the clip, False otherwise
        """
        clip_id = self._clip_id or clip_id

        if clip_id is None:
            raise ParameterError("clip_id is required")

        body = {"clipId": clip_id, "noteId": note_id}
        result: bool = await self._session.request(
            Route("POST", "/api/clips/remove-note"), json=body, auth=True
        )
        return result

    async def delete(self, clip_id: str | None = None) -> bool:
        """Delete a clip

        Parameters
        ----------
        clip_id : str | None, optional, by default None
            The clip id

        Returns
        -------
        bool
            True if the clip was deleted, False otherwise
        """
        clip_id = self._clip_id or clip_id

        if clip_id is None:
            raise ParameterError("clip_id is required")

        body = {"clipId": clip_id}
        result: bool = await self._session.request(
            Route("POST", "/api/clips/delete"), json=body, auth=True
        )
        return result

    async def update(
        self,
        name: str,
        is_public: bool | None = None,
        description: str | None = None,
        clip_id: str | None = None,
    ) -> Clip:
        """Update a clip

        Parameters
        ----------
        clip_id : str | None, optional, by default None
            The clip id
        name : str
            The clip name
        is_public : bool, optional
            Whether the clip is public, by default None
        description : str, optional
            The clip description, by default None

        Returns
        -------
        bool
            True if the clip was updated, False otherwise
        """

        clip_id = self._clip_id or clip_id

        if clip_id is None:
            raise ParameterError("clip_id is required")

        body = {"clipId": clip_id, "name": name, "isPublic": is_public, "description": description}
        result: IClip = await self._session.request(
            Route("POST", "/api/clips/update"), json=body, auth=True
        )
        return Clip(result, client=self._client)


class ClipActions(ClientClipActions):
    def __init__(self, *, clip_id: str | None = None, session: HTTPClient, client: ClientManager):
        super().__init__(clip_id=clip_id, session=session, client=client)

    async def get_my_favorites(self):
        """Get my favorite clips

        Returns
        -------
        list[Clip]
            The favorite clips
        """
        clips: list[INote] = await self._session.request(
            Route("POST", "/api/clips/my-favorites"), auth=True
        )
        return [Note(raw_clip, client=self._client) for raw_clip in clips]

    async def create(
        self, name: str, is_public: bool = False, description: str | None = None
    ) -> Clip:
        """Create a clip

        Parameters
        ----------
        name : str
            The clip name
        is_public : bool, optional
            Whether the clip is public, by default False
        description : str, optional
            The clip description, by default None

        Returns
        -------
        Clip
            The created clip
        """
        body = {"name": name, "isPublic": is_public, "description": description}
        clip: IClip = await self._session.request(
            Route("POST", "/api/clips/create"), json=body, auth=True
        )
        return Clip(clip, client=self._client)

    async def get_list(self) -> list[Clip]:
        """Get my clips

        Returns
        -------
        list[Clip]
            The clips
        """
        clips: list[IClip] = await self._session.request(
            Route("POST", "/api/clips/list"), auth=True
        )
        return [Clip(raw_clip, client=self._client) for raw_clip in clips]

    async def get(self, clip_id: str) -> Clip:
        """Get a clip

        Parameters
        ----------
        clip_id : str
            The clip id

        Returns
        -------
        Clip
            The clip
        """
        body = {"clipId": clip_id}
        clip: IClip = await self._session.request(
            Route("POST", "/api/clips/show"), json=body, auth=True
        )
        return Clip(clip, client=self._client)



from typing import TypedDict


class IAnnouncementCommon(TypedDict):
    id: str
    created_at: str
    updated_at: str | None
    text: str
    title: str
    image_url: str | None


class IAnnouncement(IAnnouncementCommon):
    """ユーザーから見たアナウンスの状態"""

    is_read: bool


class IAnnouncementSystem(IAnnouncementCommon):
    """システムから見たアナウンスの状態"""

    reads: int



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Generic, TypeVar

from mipac.abstract.model import AbstractModel
from mipac.types.announcement import IAnnouncement, IAnnouncementCommon, IAnnouncementSystem
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.actions.admins.announcement import AdminAnnouncementClientActions
    from mipac.manager.client import ClientManager

T = TypeVar("T", bound=IAnnouncementCommon)


class AnnouncementCommon(Generic[T], AbstractModel):
    def __init__(self, announcement: T, *, client: ClientManager) -> None:
        self.__announcement: T = announcement
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__announcement["id"]

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self.__announcement["created_at"])

    @property
    def updated_at(self) -> datetime | None:
        return (
            str_to_datetime(self.__announcement["updated_at"])
            if self.__announcement["updated_at"]
            else None
        )

    @property
    def text(self) -> str:
        return self.__announcement["text"]

    @property
    def title(self) -> str:
        return self.__announcement["title"]

    @property
    def image_url(self) -> str | None:
        return self.__announcement["image_url"]

    @property
    def action(self) -> AdminAnnouncementClientActions:
        return self.__client.admin.announcement._create_client_announcement_instance(
            announce_id=self.id
        )

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Announcement) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class Announcement(AnnouncementCommon):
    def __init__(self, announcement: IAnnouncement, *, client: ClientManager) -> None:
        super().__init__(announcement, client=client)
        self.__announcement: IAnnouncement

    @property
    def is_read(self) -> bool:
        return self.__announcement["is_read"]


class AnnouncementSystem(AnnouncementCommon):
    def __init__(self, announcement: IAnnouncementSystem, *, client: ClientManager) -> None:
        super().__init__(announcement, client=client)
        self.__announcement: IAnnouncementSystem

    @property
    def reads(self) -> int:
        return self.__announcement["reads"]



from __future__ import annotations

from typing import TYPE_CHECKING, NotRequired, TypeGuard, TypedDict

if TYPE_CHECKING:
    from mipac.types.user import IMeDetailed, IUserDetailed


class IRoleUser(TypedDict):
    id: str
    user: IUserDetailed
    expires_at: str | None


class IMeRole(TypedDict):
    id: str
    user: IMeDetailed
    expires_at: str | None


class IRolePolicieValue(TypedDict):
    value: int
    use_default: bool
    priority: NotRequired[int]


class IRolePolicies(TypedDict):
    antenna_limit: IRolePolicieValue
    gtl_available: IRolePolicieValue
    ltl_available: IRolePolicieValue
    can_public_note: IRolePolicieValue
    drive_capacity_mb: IRolePolicieValue
    can_invite: IRolePolicieValue
    can_manage_custom_emojis: IRolePolicieValue
    can_hide_ads: IRolePolicieValue
    pin_limit: IRolePolicieValue
    word_mute_limit: IRolePolicieValue
    webhook_limit: IRolePolicieValue
    clip_limit: IRolePolicieValue
    note_each_clips_limit: IRolePolicieValue
    user_list_limit: IRolePolicieValue
    user_each_user_lists_limit: IRolePolicieValue
    rate_limit_factor: IRolePolicieValue


class IRole(TypedDict):
    id: str
    created_at: str
    updated_at: str
    name: str
    description: str
    color: str | None
    icon_url: str | None
    target: str
    cond_formula: dict
    is_public: bool
    is_administrator: bool
    is_moderator: bool
    as_badge: bool
    can_edit_members_by_moderator: bool
    policies: IRolePolicies
    users_count: int


def is_me_role(data: IMeRole | IRoleUser, me_id: str) -> TypeGuard[IMeRole]:
    return data["user"]["id"] == me_id and data["user"].get("avatar_id") is not None



from typing import Literal, TypedDict


class IAds(TypedDict):
    id: str
    ratio: int
    place: str
    url: str
    image_url: str


class IAd(TypedDict):
    id: str
    created_at: str
    starts_at: int
    expires_at: int
    url: str
    place: Literal["square" "horizontal" "horizontal-big"]
    priority: Literal["high" "middle" "low"]
    ratio: int
    image_url: str
    memo: str | None



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.channel import ChannelActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ChannelManager(AbstractManager):
    def __init__(
        self, channel_id: str | None = None, *, session: HTTPClient, client: ClientManager
    ):
        self.__channel_id: str | None = channel_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> ChannelActions:
        return ChannelActions(
            channel_id=self.__channel_id, session=self.__session, client=self.__client
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.admins.announcement import (
    AdminAnnouncementActions,
    AdminAnnouncementClientActions,
)
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class AdminAnnouncementManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> AdminAnnouncementActions:
        return AdminAnnouncementActions(session=self.__session, client=self.__client)

    def _create_client_announcement_instance(
        self, announce_id: str
    ) -> AdminAnnouncementClientActions:
        return AdminAnnouncementClientActions(
            announce_id=announce_id,
            session=self.__session,
            client=self.__client,
        )



from functools import lru_cache
from typing import Any

DEFAULT_CACHE: dict[str, list[str]] = {}
DEFAULT_CACHE_VALUE: dict[str, Any] = {}


def set_cache(group: str, key: str, value: Any):
    if len(DEFAULT_CACHE.get(group, [])) > 50:
        del DEFAULT_CACHE[group][-1]
        del DEFAULT_CACHE_VALUE[key]

    if DEFAULT_CACHE.get(group) is None:
        DEFAULT_CACHE[group] = []
    DEFAULT_CACHE[group].append(key)
    DEFAULT_CACHE_VALUE[key] = value


def cache(group: str = "default", override: bool = False):
    def decorator(func):
        async def wrapper(self, *args, **kwargs):
            key = cache_key_builder(func, self, *args, **kwargs)
            hit_item = DEFAULT_CACHE_VALUE.get(key)
            if hit_item and override is False and kwargs.get("cache_override") is None:
                return hit_item
            res = await func(self, *args, **kwargs)
            set_cache(group, key, res)
            return res

        return wrapper

    return decorator


def get_cache_key(func):
    async def decorator(self, *args, **kwargs):
        key = cache_key_builder(func, self, *args, **kwargs)
        return await func(self, *args, **kwargs, cache_key=key)

    return decorator


@lru_cache
def cache_key_builder(func, cls, *args, **kwargs):
    ordered_kwargs = sorted(kwargs.items())
    key = (func.__module__ or "") + ".{0}" + f"{cls}" + str(args) + str(ordered_kwargs)
    return key



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.clip import ClipActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientClipManager(AbstractManager):
    def __init__(
        self,
        clip_id: str | None = None,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        self.__clip_id = clip_id
        self.__session = session
        self.__client = client

    @property
    def action(self) -> ClipActions:
        return ClipActions(
            clip_id=self.__clip_id,
            session=self.__session,
            client=self.__client,
        )


class ClipManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session = session
        self.__client = client

    @property
    def action(self) -> ClipActions:
        return ClipActions(session=self.__session, client=self.__client)

    def _get_client_clip_instance(self, *, clip_id: str | None = None) -> ClientClipManager:
        return ClientClipManager(clip_id=clip_id, session=self.__session, client=self.__client)



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Self

from mipac.abstract.model import AbstractModel
from mipac.models.lite.note import PartialNote
from mipac.models.lite.user import LiteUser
from mipac.models.poll import Poll
from mipac.types.note import (
    INote,
    INoteReaction,
    INoteState,
    INoteTranslateResult,
    INoteUpdated,
    INoteUpdatedDelete,
)
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.types.emoji import ICustomEmojiLite

__all__ = (
    "NoteState",
    "Note",
    "Header",
    "NoteReaction",
    "NoteDeleted",
    "NoteTranslateResult",
)


class NoteState(AbstractModel):
    def __init__(self, data: INoteState) -> None:
        self.__data: INoteState = data

    @property
    def is_favorite(self) -> bool:
        return self.__data["is_favorited"]

    @property
    def is_watching(self) -> bool:
        return self.__data["is_watching"]

    @property
    def is_muted_thread(self) -> bool:
        return self.__data.get("is_muted_thread", False)


class NoteDeleted(AbstractModel):
    def __init__(self, data: INoteUpdated[INoteUpdatedDelete]) -> None:
        self.__data = data

    @property
    def note_id(self) -> str:
        return self.__data["body"]["id"]

    @property
    def deleted_at(self) -> datetime:
        return str_to_datetime(self.__data["body"]["body"]["deleted_at"])

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, NoteDeleted) and self.note_id == __value.note_id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class Header:
    def __init__(self, data):
        self.id = data.get("id")
        self.type = data.get("type")


class NoteReaction(AbstractModel):
    __slots__ = ("__reaction", "__client")

    def __init__(self, reaction: INoteReaction, *, client: ClientManager):
        self.__reaction: INoteReaction = reaction
        self.__client: ClientManager = client

    @property
    def id(self) -> str | None:
        return self.__reaction["id"]

    @property
    def created_at(self) -> datetime | None:
        return (
            datetime.strptime(self.__reaction["created_at"], "%Y-%m-%dT%H:%M:%S.%fZ")
            if "created_at" in self.__reaction
            else None
        )

    @property
    def type(self) -> str | None:
        return self.__reaction["type"]

    @property
    def user(self) -> LiteUser:
        return LiteUser(self.__reaction["user"], client=self.__client)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, NoteReaction) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class Note(PartialNote[INote]):
    """
    Noteモデル

    Parameters
    ----------
    note: INote
        The raw data of the note

    client: ClientManager
    """

    def __init__(self, note: INote, client: ClientManager):
        super().__init__(note_data=note, client=client)

    @property
    def emojis(self) -> list[ICustomEmojiLite]:  # TODO: モデルに
        """
        Note text contains a list of emojis
        Note: emojis have been abolished since misskey v13

        Returns
        -------
        list[ICustomEmojiLite]
            List of emojis contained in note text
        """

        return self._note.get("emojis", [])

    @property
    def renote(self) -> Self | None:
        return (
            Note(note=self._note["renote"], client=self._client)
            if "renote" in self._note
            else None
        )

    @property
    def reply(self) -> Self | None:
        return (
            Note(note=self._note["reply"], client=self._client) if "reply" in self._note else None
        )

    @property
    def visible_user_ids(self) -> list[str]:
        return self._note["visible_user_ids"] if "visible_user_ids" in self._note else []

    @property
    def local_only(self) -> bool:
        return self._note["local_only"] if "local_only" in self._note else False

    @property
    def my_reaction(self) -> str | None:
        return self._note["my_reaction"] if "my_reaction" in self._note else None

    @property
    def uri(self) -> str | None:
        return self._note["uri"] if "uri" in self._note else None

    @property
    def url(self) -> str | None:
        return self._note["url"] if "url" in self._note else None

    @property
    def is_hidden(self) -> bool:
        return self._note["is_hidden"] if "is_hidden" in self._note else False

    @property
    def poll(self) -> Poll | None:
        return Poll(self._note["poll"], client=self._client) if "poll" in self._note else None


class NoteTranslateResult(AbstractModel):
    """
    NoteTranslateResult

    Parameters
    ----------
    translate_result: INoteTranslateResult
        The raw data of the note translate result
    """

    def __init__(self, translate_result: INoteTranslateResult):
        self.__translate_result = translate_result

    @property
    def source_language(self) -> str:
        return self.__translate_result["sourceLang"]

    @property
    def text(self) -> str:
        return self.__translate_result["text"]



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.user import BlockingUser, UserDetailed
from mipac.types.user import IBlockingUser, IUserDetailed
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.client import ClientManager


class BlockingActions(AbstractAction):
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__user_id: str | None = user_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def add(self, user_id: str | None = None) -> UserDetailed:
        user_id = self.__user_id or user_id
        res: IUserDetailed = await self.__session.request(
            Route("POST", "/api/blocking/create"), auth=True, json={"userId": user_id}, lower=True
        )
        return UserDetailed(res, client=self.__client)

    async def remove(self, user_id: str | None = None) -> UserDetailed:
        user_id = self.__user_id or user_id
        res: IUserDetailed = await self.__session.request(
            Route("POST", "/api/blocking/delete"), auth=True, json={"userId": user_id}, lower=True
        )
        return UserDetailed(res, client=self.__client)

    async def get_list(
        self,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 100,
        get_all: bool = False,
    ) -> AsyncGenerator[BlockingUser, None]:
        if get_all:
            limit = 100

        data = {"limit": limit, "sinceId": since_id, "untilId": until_id}

        pagination = Pagination[IBlockingUser](
            self.__session, Route("POST", "/api/blocking/list"), json=data
        )

        while True:
            res_users = await pagination.next()
            for user in res_users:
                yield BlockingUser(user, client=self.__client)
            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.models.lite.user import LiteUser
from mipac.types.clip import IClip

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.manager.clip import ClientClipManager


class Clip(AbstractModel):
    def __init__(self, clip_data: IClip, *, client: ClientManager) -> None:
        self.__clip: IClip = clip_data
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """The clip id"""
        return self.__clip["id"]

    @property
    def created_at(self) -> str:
        """The time the clip was created"""
        return self.__clip["created_at"]

    @property
    def last_clipped_at(self) -> str:
        """The last time the clip was clipped"""
        return self.__clip["last_clipped_at"]

    @property
    def user_id(self) -> str:
        """The user id who created the clip"""
        return self.__clip["user_id"]

    @property
    def user(self) -> LiteUser:
        """The user who created the clip"""
        return LiteUser(self.__clip["user"], client=self.__client)

    @property
    def name(self) -> str:
        """The clip name"""
        return self.__clip["name"]

    @property
    def description(self) -> str | None:
        """The clip description"""
        return self.__clip["description"]

    @property
    def is_public(self) -> bool:
        """Whether the clip is public"""
        return self.__clip["is_public"]

    @property
    def is_favorited(self) -> bool:
        """Whether the clip is favorited"""
        return self.__clip["is_favorited"]

    @property
    def favorited_count(self) -> int:
        """The number of times the clip has been favorited"""
        return self.__clip["favorited_count"]

    @property
    def api(self) -> ClientClipManager:
        return self.__client.clip._get_client_clip_instance(clip_id=self.id)



from __future__ import annotations

import json
import logging
import re
import sys
from typing import Any, Literal, TypeVar

import aiohttp

from mipac import __version__
from mipac.config import config
from mipac.errors.base import APIError
from mipac.types.endpoints import ENDPOINTS
from mipac.types.meta import IMeta
from mipac.types.user import IMeDetailed
from mipac.utils.format import remove_dict_empty, upper_to_lower
from mipac.utils.util import COLORS, _from_json

_log = logging.getLogger(__name__)


class _MissingSentinel:
    def __eq__(self, other):
        return False

    def __bool__(self):
        return False

    def __repr__(self):
        return "..."


MISSING: Any = _MissingSentinel()
R = TypeVar("R")


class MisskeyClientWebSocketResponse(aiohttp.ClientWebSocketResponse):
    async def close(self, *, code: int = 4000, message: bytes = b"") -> bool:
        return await super().close(code=code, message=message)


async def json_or_text(response: aiohttp.ClientResponse):
    text = await response.text(encoding="utf-8")
    try:
        if "application/json" in response.headers["Content-Type"]:
            return _from_json(text)
    except KeyError:
        pass


class Route:
    def __init__(self, method: Literal["GET", "POST"], path: ENDPOINTS):
        self.path: str = path
        self.method: str = method


class HTTPClient:
    def __init__(self, url: str, token: str | None = None) -> None:
        user_agent = (
            "Misskey Bot (https://github.com/yupix/MiPA {0})" + "Python/{1[0]}.{1[1]} aiohttp/{2}"
        )
        self.user_agent = user_agent.format(__version__, sys.version_info, aiohttp.__version__)
        self._session: aiohttp.ClientSession = MISSING
        self._url: str = url
        self._token: str | None = token

    @property
    def session(self) -> aiohttp.ClientSession:
        return self._session

    async def request(
        self,
        route: Route,
        auth: bool = False,
        remove_none: bool = True,
        lower: bool = True,
        **kwargs,
    ) -> R:
        headers: dict[str, str] = {
            "User-Agent": self.user_agent,
        }

        if "json" in kwargs:
            headers["Content-Type"] = "application/json"
            kwargs["json"] = kwargs.pop("json")

        if auth:
            key = "json" if "json" in kwargs or "data" not in kwargs else "data"
            if not kwargs.get(key):
                kwargs[key] = {}
            if self._token:
                kwargs[key]["i"] = self._token

        replace_list = kwargs.pop("replace_list", {})

        for i in ("json", "data"):
            if kwargs.get(i) and remove_none:
                kwargs[i] = remove_dict_empty(kwargs[i])
        async with self._session.request(route.method, self._url + route.path, **kwargs) as res:
            data = await json_or_text(res)
            if lower:
                if isinstance(data, list):
                    data = [upper_to_lower(i, replace_list=replace_list) for i in data]
                if isinstance(data, dict):
                    data = upper_to_lower(data)
            _log.debug(
                f"""{COLORS.green}
REQUEST:{COLORS.reset}
    {kwargs}
{COLORS.green}RESPONSE:{COLORS.reset}
    {json.dumps(data, ensure_ascii=False, indent=4) if data else data}
                """
            )
            if res.status == 204 and data is None:
                return True  # type: ignore
            if 300 > res.status >= 200:
                return data  # type: ignore
            if 511 > res.status >= 300:
                if isinstance(data, dict):
                    APIError(data, res.status).raise_error()
            APIError("HTTP ERROR", res.status).raise_error()

    async def close_session(self) -> None:
        await self._session.close()

    async def login(self) -> IMeDetailed | None:
        match_domain = re.search(r"https?:\/\/([^\/]+)", self._url)
        match_protocol = re.search(r"^(http|https)", self._url)
        if match_domain is None or match_protocol is None:
            raise Exception("Server URL cannot be retrieved or protocol (http / https) is missing")
        protocol = True if match_protocol.group(1) == "https" else False
        config.from_dict(
            host=match_domain.group(1),
            is_ssl=protocol,
        )
        self._session = aiohttp.ClientSession(ws_response_class=MisskeyClientWebSocketResponse)
        if self._token:
            data: IMeDetailed = await self.request(Route("POST", "/api/i"), auth=True)
            if config.use_version_autodetect:
                meta: IMeta = await self.request(Route("POST", "/api/meta"), auth=True)
                use_version = int(meta["version"].split(".")[0])
                if isinstance(use_version, int) and use_version in (13, 12, 11):
                    config.use_version = use_version
            config.from_dict(account_id=data["id"])
            return data



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.admins.admin import AdminActions
from mipac.http import HTTPClient
from mipac.manager.admins.ad import AdminAdvertisingManager, AdminAdvertisingModelManager
from mipac.manager.admins.announcement import AdminAnnouncementManager
from mipac.manager.admins.emoji import AdminEmojiManager
from mipac.manager.admins.invite import AdminInviteManager
from mipac.manager.admins.moderator import AdminModeratorManager
from mipac.manager.admins.roles import AdminRolesManager, AdminRolesModelManager
from mipac.manager.admins.user import AdminUserManager

if TYPE_CHECKING:
    from mipac.client import ClientManager


class AdminManager(AbstractManager):
    def __init__(self, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.emoji: AdminEmojiManager = AdminEmojiManager(session=session, client=client)
        self.user: AdminUserManager = AdminUserManager(session=session, client=client)
        self.ad: AdminAdvertisingManager = AdminAdvertisingManager(session=session, client=client)
        self.moderator: AdminModeratorManager = AdminModeratorManager(
            session=session, client=client
        )
        self.announcement: AdminAnnouncementManager = AdminAnnouncementManager(
            session=session, client=client
        )
        self.role: AdminRolesManager = AdminRolesManager(session=session, client=client)
        self.invite: AdminInviteManager = AdminInviteManager(session=session, client=client)

    @property
    def action(self) -> AdminActions:
        return AdminActions(session=self.__session, client=self.__client)

    def create_roles_model_manager(self, role_id: str | None = None) -> AdminRolesModelManager:
        return AdminRolesModelManager(
            role_id=role_id, session=self.__session, client=self.__client
        )

    def create_ad_model_manager(self, ad_id: str | None = None) -> AdminAdvertisingModelManager:
        return AdminAdvertisingModelManager(
            ad_id=ad_id, session=self.__session, client=self.__client
        )



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.errors.base import ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.mute import MuteUser
from mipac.types.mute import IMuteUser
from mipac.utils.format import remove_dict_empty
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class MuteActions(AbstractAction):
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self._user_id: str | None = user_id
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def add(self, user_id: str | None = None) -> bool:
        """
        Adds the specified user as a mute target

        Parameters
        ----------
        user_id : str | None, optional
            Mute target user Id, by default None

        Returns
        -------
        bool
            Whether the mute was successful or not
        """

        user_id = user_id or self._user_id
        data = {"userId": user_id}
        res: bool = await self._session.request(
            Route("POST", "/api/mute/create"), auth=True, json=data
        )
        return res

    async def delete(self, user_id: str | None = None) -> bool:
        """
        Unmute the specified user

        Parameters
        ----------
        user_id : str | None, optional
            Unmute target user Id, by default None

        Returns
        -------
        bool
            Whether the unmute was successful or not.
        """

        user_id = user_id or self._user_id
        data = {"userId": user_id}
        res: bool = await self._session.request(
            Route("POST", "/api/mute/delete"), auth=True, json=data
        )
        return res

    async def gets(
        self,
        limit: int = 100,
        since_id: str | None = None,
        until_id: str | None = None,
        get_all: bool = True,
    ) -> AsyncGenerator[MuteUser, None]:
        if limit > 100:
            raise ParameterError("limit は100以下である必要があります")

        if get_all:
            limit = 100

        body = remove_dict_empty({"limit": limit, "sinceId": since_id, "untilId": until_id})

        pagination = Pagination[IMuteUser](
            self._session, Route("POST", "/api/mute/list"), json=body
        )

        while True:
            raw_mute_users = await pagination.next()
            for raw_mute_user in raw_mute_users:
                yield MuteUser(raw_mute_user, client=self._client)

            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager

__all__ = ("AbstractAction",)


class AbstractAction(ABC):
    @abstractmethod
    def __init__(self, session: HTTPClient, client: ClientManager):
        pass



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.note import ClientNoteActions, NoteActions
from mipac.http import HTTPClient
from mipac.manager.favorite import FavoriteManager
from mipac.manager.poll import PollManager
from mipac.manager.reaction import ReactionManager

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientNoteManager(AbstractManager):
    def __init__(self, note_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__note_id = note_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.reaction: ReactionManager = ReactionManager(
            note_id=note_id, session=session, client=client
        )
        self.favorite = FavoriteManager(note_id=note_id, session=session, client=client)
        self.poll: PollManager = PollManager(note_id=note_id, session=session, client=client)

    @property
    def action(self) -> ClientNoteActions:
        return ClientNoteActions(
            note_id=self.__note_id,
            session=self.__session,
            client=self.__client,
        )


class NoteManager(AbstractManager):
    """User behavior for notes"""

    def __init__(self, note_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__note_id: str | None = note_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.reaction: ReactionManager = ReactionManager(
            note_id=note_id, session=session, client=client
        )
        self.favorite = FavoriteManager(note_id=note_id, session=session, client=client)
        self._client: ClientNoteManager = ClientNoteManager(
            note_id=note_id, session=session, client=client
        )
        self.poll: PollManager = PollManager(note_id=note_id, session=session, client=client)

    def create_client_note_manager(self, note_id: str) -> ClientNoteManager:
        return ClientNoteManager(note_id=note_id, session=self.__session, client=self.__client)

    @property
    def action(self) -> NoteActions:
        return NoteActions(
            note_id=self.__note_id,
            session=self.__session,
            client=self.__client,
        )



from __future__ import annotations

from typing import TYPE_CHECKING, TypedDict

if TYPE_CHECKING:
    from mipac.types.user import IUserDetailed


class IUserIP(TypedDict):
    ip: str
    created_at: str


class IIndexStat(TypedDict):
    schemaname: str
    tablename: str
    indexname: str
    tablespace: str | None
    indexdef: str


class ITableStats(TypedDict):
    count: int
    size: int


class IModerationLog(TypedDict):
    id: str
    created_at: str
    type: str
    info: dict  # TODO: これ何?
    user_id: str
    user: IUserDetailed


class IServerInfoCpu(TypedDict):
    models: str
    cores: int


class IServerInfoMem(TypedDict):
    total: int


class IServerInfoFs(TypedDict):
    total: int
    used: int


class IServerInfoNet(TypedDict):
    interface: str


class IServerInfo(TypedDict):
    machine: str
    os: str
    node: str
    psql: str
    cpu: IServerInfoCpu
    mem: IServerInfoMem
    fs: IServerInfoFs
    net: IServerInfoNet



from typing import TypedDict

__all__ = ("IPoll", "IPollChoice", "ICreatePoll", "IBasePoll")


class IPollChoice(TypedDict):
    is_voted: bool
    text: str
    votes: int


class IBasePoll(TypedDict, total=False):
    multiple: bool
    expires_at: int
    expired_after: int


class ICreatePoll(IBasePoll, total=False):
    choices: list[str]


class IPoll(IBasePoll):
    """
    Questionnaire object
    """

    choices: list[IPollChoice]



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.errors.base import ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.note import Note
from mipac.types.note import INote
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class PollActions(AbstractAction):
    def __init__(self, note_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__note_id: str | None = note_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def vote(self, choice: int, note_id: str | None = None) -> bool:
        note_id = note_id or self.__note_id

        if note_id is None:
            raise ParameterError("note_id is required")

        data = {"noteId": note_id, "choice": choice}
        res: bool = await self.__session.request(
            Route("POST", "/api/notes/polls/vote"), auth=True, json=data
        )
        return res

    async def recommendation(self, limit: int = 100, offset: int = 0, get_all: bool = True):
        if limit > 100:
            raise ParameterError("limit must be less than 100")

        if get_all:
            limit = 100

        data = {"limit": limit, "offset": offset}

        pagination = Pagination[INote](
            self.__session,
            Route("POST", "/api/notes/polls/recommendation"),
            json=data,
            pagination_type="count",
        )

        while True:
            res_notes = await pagination.next()
            for note in res_notes:
                yield Note(note, client=self.__client)
            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.errors.base import NotSupportVersion, NotSupportVersionText
from mipac.http import HTTPClient, Route
from mipac.models.emoji import CustomEmoji
from mipac.types.emoji import ICustomEmoji

if TYPE_CHECKING:
    from mipac.client import ClientManager


class EmojiActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def get(self, name: str) -> CustomEmoji:
        if self.__client._config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)
        emoji: ICustomEmoji = await self.__session.request(
            Route("POST", "/api/emoji"), auth=True, lower=True, json={"name": name}
        )
        return CustomEmoji(emoji=emoji, client=self.__client)



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Generic, Literal, TypeVar

from mipac.abstract.model import AbstractModel
from mipac.models.announcement import Announcement
from mipac.models.lite.user import BadgeRole, LiteUser
from mipac.models.note import Note
from mipac.types.page import IPage
from mipac.types.user import (
    IAchievement,
    IBlockingUser,
    IMeDetailed,
    IUserDetailed,
    IUserDetailedField,
    IUserRole,
)
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager

__all__ = ("UserDetailed", "LiteUser", "Achievement", "BlockingUser", "MeDetailed")

T = TypeVar("T", bound=IUserDetailed)


class BlockingUser(AbstractModel):
    def __init__(self, blocking_user_data: IBlockingUser, *, client: ClientManager) -> None:
        self.__blocking_user_data: IBlockingUser = blocking_user_data
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__blocking_user_data["id"]

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self.__blocking_user_data["created_at"])

    @property
    def blockee_id(self) -> str:
        return self.__blocking_user_data["blockee_id"]

    @property
    def blockee(self) -> UserDetailed:
        return UserDetailed(self.__blocking_user_data["blockee"], client=self.__client)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, BlockingUser) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class Achievement(AbstractModel):
    def __init__(self, detail: IAchievement):
        self.__detail: IAchievement = detail

    @property
    def name(self) -> str:
        return self.__detail["name"]

    @property
    def unlocked_at(self) -> int:
        return self.__detail["unlocked_at"]


class UserRole(BadgeRole[IUserRole]):
    def __init__(self, data: IUserRole, *, client: ClientManager) -> None:
        super().__init__(data, client=client)

    @property
    def id(self) -> str:
        return self._data["id"]

    @property
    def color(self) -> str | None:
        return self._data["color"]

    @property
    def description(self) -> str:
        return self._data["description"]

    @property
    def is_moderator(self) -> bool:
        return self._data["is_moderator"]

    @property
    def is_administrator(self) -> bool:
        return self._data["is_administrator"]


class UserDetailed(LiteUser[T], Generic[T]):
    __slots__ = ("__client",)

    def __init__(self, user: T, *, client: ClientManager):
        super().__init__(user=user, client=client)

    @property
    def achievements(self) -> list[Achievement]:
        return [Achievement(i) for i in self._user.get("achievements", [])]

    @property
    def fields(self) -> list[IUserDetailedField]:
        return self._user["fields"]

    @property
    def followers_count(self) -> int:
        return self._user["followers_count"]

    @property
    def following_count(self) -> int:
        return self._user["following_count"]

    @property
    def has_pending_follow_request_from_you(self) -> bool | None:
        return self._user.get("has_pending_follow_request_from_you")

    @property
    def has_pending_follow_request_to_you(self) -> bool | None:
        return self._user.get("has_pending_follow_request_to_you")

    @property
    def is_admin(self) -> bool:
        return self._user["is_admin"]

    @property
    def is_blocked(self) -> bool | None:
        return self._user.get("is_blocked")

    @property
    def is_blocking(self) -> bool | None:
        return self._user.get("is_blocking")

    @property
    def is_bot(self) -> bool:
        return self._user["is_bot"]

    @property
    def is_cat(self) -> bool:
        return self._user["is_cat"]

    @property
    def is_followed(self) -> bool | None:
        return self._user.get("is_followed")

    @property
    def is_following(self) -> bool | None:
        return self._user.get("is_following")

    @property
    def is_locked(self) -> bool:
        return self._user["is_locked"]

    @property
    def is_moderator(self) -> bool:
        return self._user["is_moderator"]

    @property
    def is_muted(self) -> bool | None:
        return self._user.get("is_muted")

    @property
    def is_silenced(self) -> bool:
        return self._user["is_silenced"]

    @property
    def is_suspended(self) -> bool:
        return self._user["is_suspended"]

    @property
    def public_reactions(self) -> bool:
        return self._user["public_reactions"]

    @property
    def security_keys(self) -> bool:
        return self._user["security_keys"]

    @property
    def two_factor_enabled(self) -> bool:
        return self._user["two_factor_enabled"]

    @property
    def notes_count(self) -> int:
        return self._user["notes_count"]

    @property
    def pinned_note_ids(self) -> list[str]:
        return self._user["pinned_note_ids"]

    @property
    def pinned_notes(self) -> list[Note]:
        return [Note(i, client=self.__client) for i in self._user["pinned_notes"]]

    @property
    def banner_blurhash(self) -> str | None:
        return self._user.get("banner_blurhash")

    @property
    def banner_color(self) -> str | None:
        return self._user.get("banner_color")

    @property
    def banner_url(self) -> str | None:
        return self._user.get("banner_url")

    @property
    def birthday(self) -> str | None:
        return self._user.get("birthday")

    @property
    def created_at(self) -> str | None:
        return self._user.get("created_at")

    @property
    def description(self) -> str | None:
        return self._user.get("description")

    @property
    def ff_visibility(
        self,
    ) -> Literal["public", "followers", "private"] | None:
        return self._user.get("ff_visibility")

    @property
    def lang(self) -> str | None:
        return self._user.get("lang")

    @property
    def last_fetched_at(self) -> str | None:
        return self._user.get("last_fetched_at")

    @property
    def location(self) -> str | None:
        return self._user.get("location")

    @property
    def logged_in_days(self) -> int | None:
        return self._user.get("logged_in_days")

    @property
    def pinned_page(self) -> IPage | None:
        return self._user.get("pinned_page")

    @property
    def pinned_page_id(self) -> str | None:
        return self._user.get("pinned_page_id")

    @property
    def updated_at(self) -> str | None:
        return self._user.get("updated_at")

    @property
    def uri(self) -> str | None:
        return self._user.get("uri")

    @property
    def url(self) -> str | None:
        return self._user.get("url")

    @property
    def use_password_less_login(self) -> bool | None:
        return self._user.get("use_password_less_login")

    @property
    def roles(self) -> list[UserRole]:
        return [UserRole(i, client=self.__client) for i in self._user.get("roles", [])]

    @property
    def memo(self) -> str | None:
        return self._user.get("memo")

    @property
    def moderation_note(self) -> str | None:
        return self._user.get("moderation_note")


class MeDetailed(UserDetailed[IMeDetailed]):
    def __init__(self, data: IMeDetailed, *, client: ClientManager):
        super().__init__(data, client=client)

    @property
    def avatar_id(self) -> str:
        return self._user["avatar_id"]

    @property
    def banner_id(self) -> str:
        return self._user["banner_id"]

    @property
    def auto_accept_followed(self) -> bool:
        return self._user["auto_accept_followed"]

    @property
    def always_mark_nsfw(self) -> bool:
        return self._user["always_mark_nsfw"]

    @property
    def careful_bot(self) -> bool:
        return self._user["careful_bot"]

    @property
    def email_notification_types(self) -> list[str]:
        return self._user["email_notification_types"]

    @property
    def has_pending_received_follow_request(self) -> bool:
        return self._user["has_pending_received_follow_request"]

    @property
    def has_unread_announcement(self) -> bool:
        return self._user["has_unread_announcement"]

    @property
    def has_unread_antenna(self) -> bool:
        return self._user["has_unread_antenna"]

    @property
    def has_unread_mentions(self) -> bool:
        return self._user["has_unread_mentions"]

    @property
    def has_unread_messaging_message(self) -> bool:
        return self._user["has_unread_messaging_message"]

    @property
    def has_unread_notification(self) -> bool:
        return self._user["has_unread_notification"]

    @property
    def has_unread_specified_notes(self) -> bool:
        return self._user["has_unread_specified_notes"]

    @property
    def hide_online_status(self) -> bool:
        return self._user["hide_online_status"]

    @property
    def inject_featured_note(self) -> bool:
        return self._user["inject_featured_note"]

    @property
    def integrations(self) -> dict[str, str]:
        return self._user["integrations"]

    @property
    def is_deleted(self) -> bool:
        return self._user["is_deleted"]

    @property
    def is_explorable(self) -> bool:
        return self._user["is_explorable"]

    @property
    def muted_words(self) -> list[list[str]]:
        return self._user["muted_words"]

    @property
    def muting_notification_types(self) -> list[str]:
        return self._user["muting_notification_types"]

    @property
    def no_crawle(self) -> bool:
        return self._user["no_crawle"]

    @property
    def receive_announcement_email(self) -> bool:
        return self._user["receive_announcement_email"]

    @property
    def use_password_less_login(self) -> bool:
        return self._user["use_password_less_login"]

    @property
    def unread_announcements(self) -> list[Announcement]:
        return [
            Announcement(announcement, client=self.__client)
            for announcement in self._user["unread_announcements"]
        ]

    @property
    def two_factor_backup_codes_stock(self) -> Literal["full", "partial", "none"]:
        return self._user["two_factor_backup_codes_stock"]



import functools
import json
import warnings
from datetime import datetime, timedelta
from typing import Any

try:
    import orjson  # type: ignore
except ModuleNotFoundError:
    HAS_ORJSON = False
else:
    HAS_ORJSON = True

if HAS_ORJSON:
    _from_json = orjson.loads  # type: ignore
else:
    _from_json = json.loads


def deprecated(func):
    """
    This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used.
    """

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        warnings.simplefilter("always", DeprecationWarning)  # turn off filter
        warnings.warn(
            "Call to deprecated function {}.".format(func.__name__),
            category=DeprecationWarning,
            stacklevel=2,
        )
        warnings.simplefilter("default", DeprecationWarning)  # reset filter
        return func(*args, **kwargs)

    return new_func


def check_multi_arg(*args: Any) -> bool:
    """
    複数の値を受け取り値が存在するかをboolで返します

    Parameters
    ----------
    args : list
        確認したい変数のリスト

    Returns
    -------
    bool
        存在する場合はTrue, 存在しない場合はFalse
    """
    return bool([i for i in args if i])


class MiTime:
    def __init__(self, start: timedelta, end: datetime):
        self.start = start
        self.end = end


class Colors:
    def __init__(self) -> None:
        self.green = "\x1b[92;1m"
        self.reset = "\x1b[0m"


COLORS = Colors()



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator

from mipac.config import config
from mipac.abstract.action import AbstractAction
from mipac.errors.base import NotSupportVersion
from mipac.http import HTTPClient, Route
from mipac.models.note import Note
from mipac.models.roles import MeRole, Role, RoleUser
from mipac.types.note import INote
from mipac.types.roles import IMeRole, IRole, IRoleUser, is_me_role
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.client import ClientManager


class RoleActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def get_list(self) -> list[Role]:
        if config.use_version < 13:
            raise NotSupportVersion("This method is only available in version 13 or later")

        res: list[IRole] = await self.__session.request(
            Route("POST", "/api/roles/list"), auth=True
        )

        return [Role(raw_role, client=self.__client) for raw_role in res]

    async def get(self, role_id: str) -> Role:
        """Get a role from the API.

        Parameters
        ----------
        role_id : str
            The ID of the role to get.

        Returns
        -------
        Role
            The role data.

        Raises
        ------
        NotSupportVersion
            If the version of the Misskey is less than 13.
        """

        if config.use_version < 13:
            raise NotSupportVersion("This method is only available in version 13 or later")

        raw_role: IRole = await self.__session.request(
            Route("POST", "/api/roles/show"), auth=True, json={"roleId": role_id}
        )
        return Role(raw_role, client=self.__client)

    async def get_users(
        self,
        role_id: str,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        *,
        get_all: bool = False,
    ) -> AsyncGenerator[MeRole | RoleUser, None]:
        if config.use_version < 13:
            raise NotSupportVersion("This method is only available in version 13 or later")

        if limit > 100:
            raise ValueError("Limit cannot be greater than 100")

        if get_all:
            limit = 100

        body = {"roleId": role_id, "limit": limit, "sinceId": since_id, "untilId": until_id}

        pagination = Pagination[IRoleUser | IMeRole](
            self.__session, Route("POST", "/api/roles/users"), auth=True, json=body
        )
        while True:
            raw_users = await pagination.next()
            for raw_user in raw_users:
                yield MeRole(raw_user, client=self.__client) if is_me_role(
                    raw_user, config.account_id
                ) else RoleUser(raw_user, client=self.__client)

            if pagination.is_final or get_all is False:
                break

    async def get_notes(
        self,
        role_id: str,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_data: int | None = None,
        until_data: int | None = None,
        *,
        get_all: bool = False,
    ):
        if config.use_version < 13:
            raise NotSupportVersion("This method is only available in version 13 or later")

        if limit > 100:
            raise ValueError("Limit cannot be greater than 100")

        if get_all:
            limit = 100

        body = {
            "roleId": role_id,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "sinceData": since_data,
            "untilData": until_data,
        }

        pagination = Pagination[INote](
            self.__session, Route("POST", "/api/roles/notes"), auth=True, json=body
        )
        while True:
            raw_notes = await pagination.next()
            for raw_note in raw_notes:
                yield Note(raw_note, client=self.__client)

            if pagination.is_final or get_all is False:
                break



import pathlib

from setuptools import setup

import versioneer

description = 'A Python wrapper for the Misskey API'
readme_file = pathlib.Path(__file__).parent / 'README.md'
with readme_file.open(encoding='utf-8') as fh:
    long_description = fh.read()

with open('requirements.txt', 'r') as f:
    requirements = f.read().splitlines()

extras_require = {
    'dev': ['black', 'isort', 'mypy', 'flake8', 'pre-commit', 'ruff'],
    'ci': ['flake8', 'mypy', 'ruff'],
    'speed': ['orjson'],
    'doc': ['sphinx', 'furo', 'sphinxcontrib_trio', 'sphinx-intl', 'numpydoc'],
}

packages = [
    'mipac',
    'mipac.abstract',
    'mipac.actions',
    'mipac.actions.admins',
    'mipac.errors',
    'mipac.manager',
    'mipac.manager.admins',
    'mipac.models',
    'mipac.types',
    'mipac.models.lite',
    'mipac.utils'
]

setup(
    name='mipac',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    install_requires=requirements,
    url='https://github.com/yupix/mipac',
    author='yupix',
    author_email='yupi0982@outlook.jp',
    license='MIT',
    python_requires='>=3.11, <4.0',
    description=description,
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=packages,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python :: 3.11',
        'Natural Language :: Japanese',
        'License :: OSI Approved :: MIT License',
    ],
    extras_require=extras_require,
)



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING

from mipac.abstract.model import AbstractModel
from mipac.types.antenna import IAntenna, IAntennaReceiveSource
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.antenna import ClientAntennaManager
    from mipac.manager.client import ClientManager


class Antenna(AbstractModel):
    def __init__(self, antenna: IAntenna, *, client: ClientManager) -> None:
        self.__antenna: IAntenna = antenna
        self.__client: ClientManager = client

    @property
    def case_sensitive(self) -> bool:
        return self.__antenna["case_sensitive"]

    @property
    def created_at(self) -> datetime:
        return str_to_datetime(self.__antenna["created_at"])

    @property
    def exclude_keywords(self) -> list[str]:
        return self.__antenna["exclude_keywords"]

    @property
    def has_unread_note(self) -> bool:
        return self.__antenna["has_unread_note"]

    @property
    def id(self) -> str:
        return self.__antenna["id"]

    @property
    def is_actor(self) -> bool:
        return self.__antenna["is_actor"]

    @property
    def keywords(self) -> list[str]:
        return self.__antenna["keywords"]

    @property
    def name(self) -> str:
        return self.__antenna["name"]

    @property
    def notify(self) -> bool:
        return self.__antenna["notify"]

    @property
    def src(self) -> IAntennaReceiveSource:
        return self.__antenna["src"]

    @property
    def user_list_id(self) -> str | None:
        return self.__antenna["user_list_id"]

    @property
    def users(self) -> list[str]:
        return self.__antenna["users"]

    @property
    def with_file(self) -> bool:
        return self.__antenna["with_file"]

    @property
    def with_replies(self) -> bool:
        return self.__antenna["with_replies"]

    @property
    def api(self) -> ClientAntennaManager:
        return self.__client.antenna._create_client_antenna_manager(antenna_id=self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Antenna) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from typing import Self

from mipac.config import Config, IMisskeyVersions, config
from mipac.http import HTTPClient
from mipac.manager.client import ClientManager
from mipac.utils.log import LOGING_LEVEL_TYPE, setup_logging


class Client:
    def __init__(
        self,
        url: str,
        token: str | None = None,
        *,
        log_level: LOGING_LEVEL_TYPE | None = "INFO",
        use_version: IMisskeyVersions = 12,
        use_version_autodetect: bool = True
    ) -> None:
        if log_level is not None:
            setup_logging(level=log_level)
        config.from_dict(use_version=use_version, use_version_autodetect=use_version_autodetect)
        self.config: Config = config
        self.http: HTTPClient = HTTPClient(url, token)

    async def __aenter__(self) -> Self:
        await self.http.login()
        return self

    async def __aexit__(self, exc_type, exc, tb) -> None:
        await self.close_session()

    @property
    def api(self) -> ClientManager:
        return ClientManager(self.http, self.config)

    async def close_session(self) -> None:
        await self.http.close_session()



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.http import Route
from mipac.models.follow import FollowRequest
from mipac.models.user import LiteUser
from mipac.types.follow import IFollowRequest

if TYPE_CHECKING:
    from mipac.http import HTTPClient
    from mipac.manager.client import ClientManager
    from mipac.types.user import ILiteUser


class FollowActions(AbstractAction):
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__user_id: str | None = user_id
        self.__session = session
        self.__client = client

    async def add(self, user_id: str | None = None) -> LiteUser:
        """
        Follow a user

        Returns
        -------
        UserLite:
            The user that you followed
        """

        user_id = user_id or self.__user_id

        data = {"userId": user_id}
        res: ILiteUser = await self.__session.request(
            Route("POST", "/api/following/create"),
            json=data,
            auth=True,
            lower=True,
        )
        return LiteUser(res, client=self.__client)

    async def remove(self, user_id: str | None = None) -> LiteUser:
        """
        Unfollow a user

        Returns
        -------
        LiteUser
            The user that you unfollowed
        """

        user_id = user_id or self.__user_id

        data = {"userId": user_id}
        res = await self.__session.request(
            Route("POST", "/api/following/delete"), json=data, auth=True
        )
        return LiteUser(res, client=self.__client)

    async def invalidate(self, user_id: str | None = None) -> LiteUser:
        """
        Make the user unfollows you

        Returns
        -------
        LiteUser
            The user that followed you
        """

        user_id = user_id or self.__user_id

        data = {"userId": user_id}
        res: ILiteUser = await self.__session.request(
            Route("POST", "/api/following/invalidate"), json=data, auth=True
        )
        return LiteUser(res, client=self.__client)


class FollowRequestActions(AbstractAction):
    def __init__(self, user_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__user_id: str | None = user_id
        self.__session = session
        self.__client = client

    async def get_all(self) -> list[FollowRequest]:
        """
        Get all follow requests

        Returns
        -------
        list[FollowRequest]
            List of follow requests
        """

        res: list[IFollowRequest] = await self.__session.request(
            Route("POST", "/api/following/requests/list"),
            auth=True,
            lower=True,
        )
        return [FollowRequest(follow_request=i, client=self.__client) for i in res]

    async def accept(self, user_id: str | None = None) -> bool:
        """
        Accept a follow request

        Parameters
        ----------
        user_id: str
            The user ID to accept

        Returns
        -------
        bool
            Whether the request was accepted
        """

        user_id = user_id or self.__user_id

        data = {"userId": user_id}
        return bool(
            await self.__session.request(
                Route("POST", "/api/following/requests/accept"),
                json=data,
                auth=True,
            )
        )

    async def reject(self, user_id: str | None = None) -> bool:
        """
        Reject a follow request

        Parameters
        ----------
        user_id: str
            The user ID to reject

        Returns
        -------
        bool
            Whether the request was rejected
        """

        user_id = user_id or self.__user_id

        data = {"userId": user_id}
        return bool(
            await self.__session.request(
                Route("POST", "/api/following/requests/reject"),
                json=data,
                auth=True,
            )
        )

    async def cancel(self, user_id: str | None = None) -> LiteUser:
        """
        Cancel a follow request

        Parameters
        ----------
        user_id: str
            The user ID to cancel

        Returns
        -------
        LiteUser
            The user that you canceled to follow
        """

        user_id = user_id or self.__user_id

        data = {"userId": user_id}
        res: ILiteUser = await self.__session.request(
            Route("POST", "/api/following/requests/cancel"),
            json=data,
            auth=True,
            lower=True,
        )
        return LiteUser(res, client=self.__client)



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, Literal, Optional, TypeVar, Union, overload

from mipac.config import config
from mipac.errors.base import (
    NotExistRequiredData,
    NotSupportVersion,
    NotSupportVersionText,
    ParameterError,
)
from mipac.http import HTTPClient, Route
from mipac.models.clip import Clip
from mipac.models.note import Note
from mipac.models.user import Achievement, LiteUser, UserDetailed
from mipac.types.clip import IClip
from mipac.types.note import INote
from mipac.types.user import ILiteUser, IUserDetailed
from mipac.utils.cache import cache
from mipac.utils.format import remove_dict_empty
from mipac.utils.pagination import Pagination, pagination_iterator
from mipac.utils.util import check_multi_arg

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager

__all__ = ["UserActions"]

T = TypeVar("T", bound=Union[LiteUser, UserDetailed])


class UserActions:
    def __init__(
        self,
        session: HTTPClient,
        client: ClientManager,
        user: Optional[LiteUser] = None,
    ):
        self.__session: HTTPClient = session
        self.__user: Optional[LiteUser] = user
        self.__client: ClientManager = client

    async def get_me(self) -> UserDetailed:
        """
        ログインしているユーザーの情報を取得します
        """

        res = await self.__session.request(
            Route("POST", "/api/i"),
            auth=True,
            lower=True,
        )
        return UserDetailed(res, client=self.__client)  # TODO: 自分用のクラスに変更する

    def get_profile_link(
        self,
        external: bool = True,
        protocol: Literal["http", "https"] = "https",
    ):
        if not self.__user:
            return None
        host = (
            f"{protocol}://{self.__user.host}"
            if external and self.__user.host
            else self.__session._url
        )
        path = (
            f"/@{self.__user.username}" if external else f"/{self.__user.api.action.get_mention()}"
        )
        return host + path

    @cache(group="get_user")
    async def get(
        self,
        user_id: str | None = None,
        username: str | None = None,
        host: str | None = None,
        **kwargs,
    ) -> UserDetailed:
        """
        Retrieve user information from the user ID using the cache.
        If there is no cache, `fetch` is automatically used.
        The `fetch` method is recommended if you want up-to-date user information.

        Parameters
        ----------
        user_id : str
            target user id
        username : str
            target username
        host : str, default=None
            Hosts with target users

        Returns
        -------
        UserDetailed
            user information
        """

        field = remove_dict_empty({"userId": user_id, "username": username, "host": host})
        data = await self.__session.request(
            Route("POST", "/api/users/show"), json=field, auth=True, lower=True
        )
        return UserDetailed(data, client=self.__client)

    async def fetch(
        self,
        user_id: str | None = None,
        username: str | None = None,
        host: str | None = None,
    ) -> UserDetailed:
        """
        Retrieve the latest user information using the target user ID or username.
        If you do not need the latest information, you should basically use the `get` method.
        This method accesses the server each time,
        which may increase the number of server accesses.

        Parameters
        ----------
        user_id : str
            target user id
        username : str
            target username
        host : str, default=None
            Hosts with target users

        Returns
        -------
        UserDetailed
            ユーザー情報
        """
        return await self.get(user_id=user_id, username=username, host=host, cache_override=True)

    async def get_notes(
        self,
        user_id: str | None = None,
        include_replies: bool = True,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int = 0,
        until_date: int = 0,
        include_my_renotes: bool = True,
        with_files: bool = False,
        file_type: Optional[list[str]] = None,
        exclude_nsfw: bool = True,
        get_all: bool = False,
    ) -> AsyncGenerator[Note, None]:
        if check_multi_arg(user_id, self.__user) is False:
            raise ParameterError("missing required argument: user_id", user_id, self.__user)

        user_id = user_id or self.__user and self.__user.id
        data = {
            "userId": user_id,
            "includeReplies": include_replies,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "sinceDate": since_date,
            "untilDate": until_date,
            "includeMyRenotes": include_my_renotes,
            "withFiles": with_files,
            "fileType": file_type,
            "excludeNsfw": exclude_nsfw,
        }

        if get_all:
            data["limit"] = 100
            limit = 100

        pagination = Pagination[INote](
            self.__session, Route("POST", "/api/users/notes"), json=data, limit=limit
        )

        while True:
            res_notes = await pagination.next()
            for note in res_notes:
                yield Note(note, client=self.__client)
            if get_all is False or pagination.is_final:
                break

    def get_mention(self, user: Optional[LiteUser] = None) -> str:
        """
        Get mention name of user.

        Parameters
        ----------
        user : Optional[User], default=None
            The object of the user whose mentions you want to retrieve

        Returns
        -------
        str
            メンション
        """

        user = user or self.__user

        if user is None:
            raise NotExistRequiredData("Required parameters: user")
        return f"@{user.username}@{user.host}" if user.instance else f"@{user.username}"

    @overload
    async def search(
        self,
        query: str,
        limit: int = 100,
        offset: int = 0,
        origin: Literal["local", "remote", "combined"] = "combined",
        detail: Literal[False] = ...,
        *,
        get_all: bool = False,
    ) -> AsyncGenerator[LiteUser, None]:
        ...

    @overload
    async def search(
        self,
        query: str,
        limit: int = 100,
        offset: int = 0,
        origin: Literal["local", "remote", "combined"] = "combined",
        detail: Literal[True] = True,
        *,
        get_all: bool = False,
    ) -> AsyncGenerator[UserDetailed, None]:
        ...

    async def search(
        self,
        query: str,
        limit: int = 100,
        offset: int = 0,
        origin: Literal["local", "remote", "combined"] = "combined",
        detail: Literal[True, False] = True,
        *,
        get_all: bool = False,
    ) -> AsyncGenerator[UserDetailed | LiteUser, None]:
        """
        Search users by keyword.

        Parameters
        ----------
        query : str
            Keyword to search.
        limit : int, default=100
            The maximum number of users to return.
        offset : int, default=0
            The number of users to skip.
        origin : Literal['local', 'remote', 'combined'], default='combined'
            The origin of users to search.
        detail : Literal[True, False], default=True
            Whether to return detailed user information.
        get_all : bool, default=False
            Whether to return all users.

        Returns
        -------
        AsyncGenerator[Union[LiteUser, UserDetailed], None]
            A AsyncGenerator of users.
        """

        if limit > 100:
            raise ParameterError("limit は100以下である必要があります")

        if get_all:
            limit = 100

        body = remove_dict_empty(
            {"query": query, "limit": limit, "offset": offset, "origin": origin, "detail": detail}
        )

        if detail:
            pagination = Pagination[IUserDetailed](
                self.__session,
                Route("POST", "/api/users/search"),
                json=body,
                pagination_type="count",
            )
            iterator = pagination_iterator(
                pagination, get_all, model=UserDetailed, client=self.__client
            )
        else:
            pagination = Pagination[ILiteUser](
                self.__session,
                Route("POST", "/api/users/search"),
                json=body,
                pagination_type="count",
            )

            iterator = pagination_iterator(
                pagination, get_all=get_all, model=LiteUser, client=self.__client
            )
        async for user in iterator:
            yield user

    async def search_by_username_and_host(
        self,
        username: str,
        host: str,
        limit: int = 100,
        detail: bool = True,
    ) -> list[UserDetailed | LiteUser]:
        """
        Search users by username and host.

        Parameters
        ----------
        username : str
            Username of user.
        host : str
            Host of user.
        limit : int, default=100
            The maximum number of users to return.
        detail : bool, default=True
            Weather to get detailed user information.

        Returns
        -------
        list[UserDetailed | LiteUser]
            A list of users.
        """

        if limit > 100:
            raise ParameterError("limit は100以下である必要があります")

        body = remove_dict_empty(
            {"username": username, "host": host, "limit": limit, "detail": detail}
        )
        res = await self.__session.request(
            Route("POST", "/api/users/search-by-username-and-host"),
            lower=True,
            auth=True,
            json=body,
        )
        return [
            UserDetailed(user, client=self.__client)
            if detail
            else LiteUser(user, client=self.__client)
            for user in res
        ]

    async def get_achievements(self, user_id: str | None = None) -> list[Achievement]:
        """Get achievements of user."""

        if config.use_version < 13:
            raise NotSupportVersion(NotSupportVersionText)

        user_id = user_id or self.__user and self.__user.id

        if not user_id:
            raise ParameterError("user_id is required")

        data = {
            "userId": user_id,
        }
        res = await self.__session.request(
            Route("POST", "/api/users/achievements"),
            json=data,
            auth=True,
            lower=True,
        )
        return [Achievement(i) for i in res]

    async def get_clips(
        self,
        user_id: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        get_all: bool = False,
    ):
        user_id = user_id or self.__user and self.__user.id

        if not user_id:
            raise ParameterError("user_id is required")

        if limit > 100:
            raise ParameterError("limit must be less than 100")

        if get_all:
            limit = 100

        body = {"userId": user_id, "limit": limit, "sinceId": since_id, "untilId": until_id}

        pagination = Pagination[IClip](
            self.__session, Route("POST", "/api/users/clips"), json=body, auth=True
        )

        while True:
            clips: list[IClip] = await pagination.next()
            for clip in clips:
                yield Clip(clip, client=self.__client)
            if get_all is False or pagination.is_final:
                break



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.admins.emoji import AdminEmojiActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.client import ClientManager


class AdminEmojiManager(AbstractManager):
    def __init__(self, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> AdminEmojiActions:
        return AdminEmojiActions(session=self.__session, client=self.__client)

    def _create_admin_emoji_instance(self, emoji_id: str) -> AdminEmojiActions:
        return AdminEmojiActions(emoji_id=emoji_id, session=self.__session, client=self.__client)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, AsyncGenerator

from mipac.abstract.action import AbstractAction
from mipac.errors.base import ParameterError
from mipac.http import HTTPClient, Route
from mipac.models.instance import FederationInstance
from mipac.models.user import UserDetailed
from mipac.types.follow import IFederationFollower, IFederationFollowing
from mipac.types.instance import IFederationInstance, IFederationInstanceStat
from mipac.types.user import IUserDetailed
from mipac.utils.pagination import Pagination

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class FederationActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session = session
        self.__client = client

    async def get_ap(self, uri: str) -> dict[Any, Any]:
        return dict(
            await self.__session.request(
                Route("POST", "/api/ap/get"), auth=True, json={"uri": uri}, lower=True
            )
        )

    async def show_ap(
        self, host: str, since_id: str | None = None, until_id: str | None = None, limit: int = 10
    ) -> FederationInstance:
        # TODO: これ本当にuntilId必要なのか確認する
        body = {"host": host, "sinceId": since_id, "untilId": until_id, "limit": limit}

        res: FederationInstance = await self.__session.request(
            Route("POST", "/api/ap/show"), auth=True, json=body
        )
        return res

    async def get_followers(
        self,
        host: str,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        get_all: bool = False,
    ):
        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")

        if get_all:
            limit = 100

        body = {"host": host, "sinceId": since_id, "untilId": until_id, "limit": limit}

        pagination = Pagination[IFederationFollower](
            self.__session, Route("POST", "/api/federation/followers"), json=body
        )

        while True:
            res_federation_followers: list[IFederationFollower] = await pagination.next()
            for federation_follower in res_federation_followers:
                yield federation_follower

            if get_all is False or pagination.is_final:
                break

    async def get_following(
        self,
        host: str,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        get_all: bool = False,
    ):
        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")

        if get_all:
            limit = 100

        body = {"host": host, "sinceId": since_id, "untilId": until_id, "limit": limit}

        pagination = Pagination[IFederationFollowing](
            self.__session, Route("POST", "/api/federation/following"), json=body
        )

        while True:
            res_federation_followings: list[IFederationFollowing] = await pagination.next()
            for federation_following in res_federation_followings:
                yield federation_following

            if get_all is False or pagination.is_final:
                break

    async def get_instances(
        self,
        host: str | None = None,
        blocked: bool | None = None,
        not_responding: bool | None = None,
        suspended: bool | None = None,
        federating: bool | None = None,
        subscribing: bool | None = None,
        publishing: bool | None = None,
        limit: int = 30,
        offset: int = 0,
        sort: str | None = None,
    ) -> list[FederationInstance]:
        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")
        body = {
            "host": host,
            "blocked": blocked,
            "notResponding": not_responding,
            "suspended": suspended,
            "federating": federating,
            "subscribing": subscribing,
            "publishing": publishing,
            "limit": limit,
            "offset": offset,
            "sort": sort,
        }

        res: list[IFederationInstance] = await self.__session.request(
            Route("POST", "/api/federation/instances"), auth=True, lower=True, json=body
        )

        return [FederationInstance(i, client=self.__client) for i in res]

    async def show_instance(self, host: str) -> FederationInstance:
        res: IFederationInstance = await self.__session.request(
            Route("POST", "/api/federation/show-instance"),
            auth=True,
            json={"host": host},
            lower=True,
        )

        return FederationInstance(res, client=self.__client)

    async def update_remote_user(self, user_id: str) -> bool:
        return bool(
            self.__session.request(
                Route("POST", "/api/federation/update-remote-user"),
                auth=True,
                json={"userId": user_id},
            )
        )

    async def get_users(
        self,
        host: str,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        get_all: bool = False,
    ) -> AsyncGenerator[UserDetailed, None]:
        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")

        if get_all:
            limit = 100

        body = {"host": host, "sinceId": since_id, "untilId": until_id, "limit": limit}

        pagination = Pagination[IUserDetailed](
            self.__session, Route("POST", "/api/federation/users"), json=body
        )

        while True:
            res_users: list[IUserDetailed] = await pagination.next()
            for user in res_users:
                yield UserDetailed(user, client=self.__client)

            if get_all is False or pagination.is_final:
                break

    async def get_stats(self, limit: int = 10) -> IFederationInstanceStat:
        if limit > 100:
            raise ParameterError("limitは100以下である必要があります")
        res: IFederationInstanceStat = await self.__session.request(
            Route("POST", "/api/federation/stats"), auth=True, body={"limit": limit}, lower=True
        )
        return res



from typing import NotRequired, TypedDict

from mipac.types.note import IPartialNote


class IPartialChannel(TypedDict):
    id: str
    name: str


class IChannelNote(IPartialNote):
    channel: IPartialChannel
    channel_id: str
    local_only: bool


class IChannelLite(IPartialChannel):
    created_at: str
    last_noted_at: str | None
    description: str | None
    user_id: str
    banner_url: str | None
    users_count: int
    notes_count: int
    pinned_note_ids: NotRequired[list[str]]  # pinned系は 13.11.0以上が必要
    pinned_notes: NotRequired[list[IChannelNote]]


class IChannel(IChannelLite):
    has_unread_note: bool
    is_following: bool | None
    is_favorited: bool | None  # is_favoritedは 13.11.0以上が必要



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.actions.chat import BaseChatAction, ChatAction
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager

__all__ = ("ChatManager",)


class ChatManager:
    def __init__(
        self,
        session: HTTPClient,
        client: ClientManager,
    ):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    @property
    def action(self) -> ChatAction:
        return ChatAction(session=self.__session, client=self.__client)

    def custom_base_chat_action(
        self, user_id: str | None = None, message_id: str | None = None
    ) -> BaseChatAction:
        return BaseChatAction(
            session=self.__session,
            client=self.__client,
            user_id=user_id,
            message_id=message_id,
        )

    def custom_action(
        self, user_id: str | None = None, message_id: str | None = None
    ) -> ChatAction:
        return ChatAction(
            session=self.__session,
            client=self.__client,
            user_id=user_id,
            message_id=message_id,
        )



from __future__ import annotations

from typing import TYPE_CHECKING, List, TypedDict

if TYPE_CHECKING:
    from mipac.types import IDriveFile
    from mipac.types.user import ILiteUser

__all__ = ("IChatGroup", "IChatMessage")


class IChatGroup(TypedDict):
    id: str
    created_at: str
    name: str
    owner_id: str
    user_ids: list[str]


class IChatMessage(TypedDict):
    id: str
    created_at: str
    file: IDriveFile
    text: str | None
    user_id: str
    user: ILiteUser
    recipient_id: str | None
    recipient: ILiteUser | None
    group_id: str
    file_id: str
    is_read: list[str]
    reads: List
    group: IChatGroup | None



import logging
from typing import Literal

LOGING_LEVEL_TYPE = Literal["NOTSET", "DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
LOGING_LEVELS = {
    "NOTSET": 0,
    "DEBUG": 10,
    "INFO": 20,
    "WARNING": 30,
    "ERROR": 40,
    "CRITICAL": 50,
}


def setup_logging(
    *,
    handler: logging.Handler | None = None,
    formatter: logging.Formatter | None = None,
    level: LOGING_LEVEL_TYPE = "INFO",
) -> None:
    if level not in LOGING_LEVELS:
        raise Exception(f"Not found logging level {level}")
    _level = LOGING_LEVELS[level]

    if handler is None:
        handler = logging.StreamHandler()

    if formatter is None:
        dt_fmt = "%Y-%m-%d %H:%M:%S"
        formatter = logging.Formatter(
            "[{asctime}] [{levelname:<8}] {name}: {message}", dt_fmt, style="{"
        )
    logger = logging.getLogger()
    handler.setFormatter(formatter)
    logger.setLevel(_level)
    logger.addHandler(handler)
